/*
 *  IBM Confidential
 *  
 *  Licensed Materials - Property of IBM
 *
 *  (C) COPYRIGHT IBM Corp. 1994-1998 All rights reserved
 *
 *  US Government Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 * Filename : QbExampl.hpp
 * Date     : 12/2/98 09:59:48
 * Version  : 1.6
 * Author   : J. Hafner
 * Purpose  : Example class -- start from here to generate a new class from
 *            scratch
 *
 *
 */

/* End of Common Block                                               */

#ifndef __QbExample_hpp
#define __QbExample_hpp

#include "QbAbsCls.hpp"

class QbExampleFeatureClass : public QbFeatureClass {

  // these variables are protected so the SubPart class
  // can see them.
protected:
  // put here any variables which are needed to hold the feature
  // vector or other class information (this example has three long's)
  long featureVector[3] ;

  // this static can be used in the constructor to make sure any
  // long winded static initialization is only done once
  static int isInitialized ;
  // we always need some place to put the default parameters for this class
  static QbParameterClass defaultParameters ;

  // add any methods here which your class needs internallly

  
public:

// from QbBaseClass
  virtual const QbInterfaceType IType() const { return FeatureInterfaceType ;}
  virtual const char *Type() const { return "QbExampleFeatureClass" ; }

// from QbFeatureClass
  virtual unsigned long FeatureSize(); 
  virtual int ToByteString(QbDatumClass &data ) ;
  virtual int FromByteString(const QbDatumClass &data ) ;

  QbExampleFeatureClass() ;

  virtual ~QbExampleFeatureClass() {} ;

  virtual int ComputeFeature(const QbGenericDataClass *data,
                             const QbParameterClass *parms ) ;

  virtual fastfloat Distance(const QbFeatureClass *feature,
                             const QbParameterClass *parms,
                             const float abortDistance ) ;

  virtual float
  ExecutionCostForDistanceFunction(const QbFeatureClass *feature,
                                   const QbParameterClass *parms,
                                   const float abortDistance );

  virtual float
  ExecutionCostForFilterFunction(QbDatabaseClass &database,
                                 QbKeyDatabaseClass *list,
                                 const unsigned long queryResultsRequested,
                                 const char *catalogName,
                                 const QbParameterClass *parms ) ;

  virtual int FilterResult(QbDatabaseClass &database,
                           QbKeyDatabaseClass *list,
                           const unsigned long queryResultsRequested,
                           const char *catalogName,
                           const QbParameterClass *parms,
                           QbKeyDatabaseClass &results ) ;

  //
  // The name of the QbGenericDataClass that this feature supports.
  //
  virtual const char *GenericDataClassname() const
     { return "QbGenericImageDataClass"; }

  //
  // The name of the QbParameterClass that this feature suppport.
  //
  virtual const char *ParameterClassname() const
    { return "QbParameterClass"; }

  virtual const QbParameterClass *GetDefaultParameters() const
    { return &defaultParameters ; }
  
  // this example has three dimensions 
  virtual const unsigned long GetDimension() const 
    { return 3 ; }  
  
  virtual int ToAsciiString( QbDatumClass &string ) ;

} ;

//
// A subpart feature class ?
//
class QbExampleSubPartFeatureClass : public QbExampleFeatureClass
{
  virtual const char *Type() const { return "QbExampleSubPartFeatureClass" ; }
  virtual Boolean IsSubPartFeature() const { return True; }
};

#endif
