/*
 * Name     : QbKeyDis.cpp
 * Author   : Xiaoming Zhu
 * Purpose  : Calculate the distance between two keys
 *
 */

#include "QbAbsCls.hpp"
#include "QbCreate.hpp"
#include "QbImgDat.hpp"
#include <limits.h>
#include <float.h>

int main( int argc, char **argv, char **envp ) {
  const QbConnectClass   *connection = NULL ;
  QbDatumClass           key ;
  QbFeatureClass         *Feature1, *Feature2;
  QbGenericImageDataClass Image;
  char                   *keyName1="flower01.jpg", *keyName2="flower02.jpg";
  char                   *catalogName="ibm33", *databaseName="../../QbicData";
  const char             *featureName="QbColorHistogramFeatureClass";
  double                 distance ;

  // Obtain a connection to database
  connection = QbGetConnection("r", databaseName);
  if(NULL == connection) {
    QbPrintErrorMessage(stderr);
    exit( 1 ) ;
  }

  Feature1 = QbCreateFeatureClass(featureName);
  Image.MakeQueryByExampleObject(catalogName, keyName1);
  Image.FromQueryString(Feature1->Type());
  Image.SetIsQuery(True);
  Feature1->ComputeFeature(&Image, Feature1->GetDefaultParameters());

  Feature2 = QbCreateFeatureClass(featureName);
  Image.MakeQueryByExampleObject(catalogName, keyName2);
  Image.FromQueryString(Feature2->Type());
  Feature2->ComputeFeature(&Image, Feature2->GetDefaultParameters());

  distance = Feature1->Distance(Feature2, Feature1->GetDefaultParameters(),
               FLT_MAX);

  printf("Distance between flower01.jpg and flower02.jpg=%lf\n", distance);

  delete Feature1;
  delete Feature2;
  QbDestroyConnection( connection ) ;
  return 0;
}
