/*
 * Name     : QbKeyIt.cpp
 * Author   : Xiaoming Zhu
 * Purpose  : Iterate the first image keys for distance
 */

#include "QbAbsCls.hpp"
#include "QbCreate.hpp"
#include "QbString.hpp"
#include "QbCatCls.hpp"
#include "QbImgDat.hpp"
#include <limits.h>
#include <float.h>
#include "quError.h"
#include "QbEnv.h"
static void Usage(char *pgm) {
  const char usage[] =
    " [-c catalog] [-d database] [-s] -f feature key1 key2\n"
    "\t-c specifies the catalog name (defaults to \"test\")\n"
    "\t-d specifies the database name (defaults to \"QbicData)\"\n"
    "\t-f feature: to use in the main (second) query\n"
    "\t-s output in short format\n";
  printf("%s %s", pgm, usage );
  exit(200);
}
int main( int argc, char **argv, char **envp ) {
  QbStringClass            tableName(128) ;
  QbDatabaseClass          *db ;
  QbDbIteratorClass        *iter = NULL;
  const QbConnectClass     *connection = NULL ;
  QbCatalogClass           *catalog=NULL;
  QbFeatureClass           *Feature1 = NULL, *Feature2 = NULL;
  QbGenericImageDataClass  Image;
  QbDatumClass             key1, value1, key2, value2;
  unsigned long            size;
  char                     *buffer;
  int                      ict, rc = 0;
  char                     *catalogName="ibm33", *databaseName="../../QbicData";
  char                     *featureName="QbColorHistogramFeatureClass";
  double                   distance ;

  QbInitializeErrorStructure(); // Initialize the QBIC error handling code

  // Obtain a connection to the database of given name
  connection = QbGetConnection("r", databaseName);
  if(NULL == connection) {
    QbPrintErrorMessage(stderr);
    goto cleanUP;
  }

  // create a catalog object inside the database
  catalog = connection->CreateCatalogClass();
  if(NULL == catalog) {
    QbPrintErrorMessage(stderr);
    goto cleanUP;
  }

  // open the catalog with a given name
  if ( catalog->OpenCatalog(catalogName) ) {
    QbPrintErrorMessage(stderr);
    goto cleanUP;
  }
  
  // create an instance of database object appropriate for the connection for
  // database operations such as Open, Close, Insert, Retrieve, etc.
  db = connection->CreateDatabaseClass() ;
  if (!db) {
     QbPrintErrorMessage(stderr); 
     goto cleanUP;
  }

  rc = catalog->GetFeatureTableName( Feature2->Type(), tableName);
  if (rc) {
     QbPrintErrorMessage(stderr); 
     goto cleanUP;
  }
  rc = db->Open( tableName ) ;
  if (rc) {
     QbPrintErrorMessage(stderr); 
     goto cleanUP;
  }

  iter = db->CreateIterator();
  if (!iter) {
     QbPrintErrorMessage(stderr); 
     goto cleanUP;
  }
  rc = iter->Next(key1, value1);
  if (rc) {
     QbPrintErrorMessage(stderr); 
     goto cleanUP;
  }
  key1.Get(size, buffer);
  printf("Query Image Key %s\n", buffer);

  Feature1 = QbCreateFeatureClass(featureName);
  if(!Feature1) {
    QbPrintErrorMessage(stderr);
    goto cleanUP;
  }
  Feature1->FromByteString(value1);

  Feature2 = QbCreateFeatureClass(featureName);
  if(!Feature2) {
    QbPrintErrorMessage(stderr);
    goto cleanUP;
  }

  for(ict=0; ict<10 && iter->More(); ict++) {
    iter->Next(key2, value2);
    Feature2->FromByteString(value2);
    distance = Feature1->Distance(Feature2, Feature1->GetDefaultParameters(),
               FLT_MAX);
    key2.Get(size, buffer);

    printf("image key=%s distance=%f\n", distance);
  }

  db->Close();

cleanUP:
  if(iter)     delete iter;
  if(Feature1) delete Feature1;
  if(Feature2) delete Feature2;
  if(catalog)  delete catalog;
  if(db)       delete db;
  if(connection) QbDestroyConnection( connection ) ;
  return 0;
}
