/*
 *  
 *  Licensed Materials - Property of IBM
 *
 *  (C) COPYRIGHT IBM Corp. 1994-1998 All rights reserved
 *
 *  US Government Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 * Filename : 66
 * Date     : 3/24/98 09:59:13
 * Version  : 1.6.3.3
 * Author   : M. Flickner, T. Donn
 * Purpose  : Interface to create/populating qbic databases
 *
 *
 */

/* End of Common Block                                               */

#ifndef __QbCatCls_hpp
#define  __QbCatCls_hpp
#include "QbString.hpp"
#include "QbAbsCls.hpp"
#include "dmbsystm.h"
class QbCatalogClass : public QbBaseClass {
  friend class QbQueryServerClass ;
private:
  int GetFeatureTableValue( QbStringClass &table, QbDatumClass &key,
                            QbDatumClass &value, QbFeatureClass *) const;
protected:
  QbStringClass catalogName;//The name of the catalog
  QbStringClass dbClassName;//The C++ class name of the database class to use
  QbDatabaseClass *db ;
  const QbConnectClass &connection ;
//
// Next three functions will become private in the future !!!
//
public: // soon to be private: !!!!!
  // return the current catalogName
  char *GetCatalogName () {return (char *) catalogName; }

  //
  // Return the name of the table/file that contains the catalog index
  //
  DMBEXPHP (int) GetFeatureCatalogName( const char *catalog,
													 QbStringClass &str ) const;
  //
  // Return the name of the table/file that contains the sampler index
  //
  DMBEXPHP (int) GetSamplerIndexName( QbStringClass &str ) const;

  //
  // Return the name of the table/file that contains the subpart catalog index
  //
  DMBEXPHP (int) GetSubPartCatalogName( const char *catalog,
													 QbStringClass &str ) const;
  //
  // Return the name of the table/file that contains the feature
  //
  DMBEXPHP(int) GetFeatureTableName(const char *feature, // name of feature
                          QbStringClass &name  // returned table name
                          ) const ;
  //
  // Return the name of the table/file that contains the feature index data
  //
  DMBEXPHP(int) GetFeatureIndexTableName(const char *feature, // name of feature
                               QbStringClass &name  // returned table name
                               ) const ;
  //
  // Return the name of the table/file that contains the distance table data
  //
  DMBEXPHP(int) GetFeatureDistTableName(const char *feature, // name of feature
                               QbStringClass &name  // returned table name
                               ) const ;
  //
  // Return the name of the table/file that contains the sampler
  //
  DMBEXPHP(int) GetSamplerTableName(const char *sampler, // name of sampler
                          QbStringClass &name  // returned table name
                          ) const ;

  DMBEXPHP (int) GetSamplerFeatureName( const char *samplerName,
													 QbDatabaseClass *db,
													 QbStringClass &data ) const;
public:
  QbCatalogClass(const QbConnectClass &) ;
  virtual ~QbCatalogClass() ;
  enum UpdateType { insertRecord=0, incrementRecord=1 , updateRecord=2,
                    forceUpdateRecord = 3 } ;

  int UpdateCheck(QbDatumClass &, QbFeatureClass *, QbDatumClass &, 
                  const UpdateType, char *) const;

  // from QbBaseClass
  virtual const QbInterfaceType IType() const { return CatalogInterfaceType ;}
  virtual const char *Type() const { return "QbCatalogClass" ; }
  //
  //Create the catalog in the persistent storage. A container will be
  //created in the persistent storage for the global catalog info such
  //as feature names, global feature information...
  virtual int CreateCatalog( const char *catalogNameArg,
									  const char *in_tablespace = NULL,
									  const char *in_indexspace = NULL );
  //
  // Drop (delete) the catalog from the persistent storage
  //
  virtual int DropCatalog ( const char *catalogNameArg ) ;
  //
  // Open the catalog for updates, inserts, deletes
  //
  virtual int OpenCatalog ( const char *catalogNameArg ) ;
  //
  // close the catalog
  //
  virtual int CloseCatalog ( ) ;
  //
  // gets database class name
  //
  virtual const char *GetDatabaseClassName();
  //
  // The following functions are valid only after the Catalog is opened.
  // Add a new feature class to the catalog A container will be created
  // in the persistent storage for the feature data
  //
  virtual int AddFeatureClass( const char *featureClassName,
										 const long  in_keySize    = -1,
										 const char *in_tablespace = NULL,
										 const char *in_indexspace = NULL ) const ;
  //
  // Delete a feature class from the catalog
  //
  virtual int DeleteFeatureClass (const char *featureClassName) const ;
  //
  // List current feature class names
  //
  virtual int ListFeatureClasses (QbDatumClass &featureList) const ;
  //
  // An overloaded function for add an input object to a set of features
  // with parameters that have already been created.
  // (modified by XMZ, June 98)
  virtual int AddRecord (const char *keyArg, // input object key
                         const QbGenericDataClass &data,
                         const QbFeatureClass **features,
                         const QbParameterClass **parms,
                         int   &numOfFeatures,
                         // update instead of insert
                         //const int update = False,
                         const UpdateType update = insertRecord) const ;
  //
  // An overloaded function for delete an input object to a set of features
  // with parameters that have already been created.
  // (modified by XMZ, June 98)
  virtual int DeleteRecord (const char *keyArg, // input object key
                            const QbFeatureClass **features,
                            int   numOfFeatures) const ;
  //
  // Add an object to the catalog Features will be computed and saved in
  // the persistent storage for all current feature classes in the
  // catalog (or in a string of space separated feature class names)
  // (modified by JLH, Nov 96)
  virtual int AddRecord (const char *keyArg, // input object key
                         const QbGenericDataClass &data,
                         // array of parameter classes - one for each
                         // feature - must be in order of features as
                         // returned by ListFeatureClasses.
                         // If NULL use feature class
                         // default parmaeter class.
                         const QbParameterClass *parms,
                         // update instead of insert
                         //const int update = False,
                         const UpdateType update = insertRecord,
                         // space-separated list of feature class names
                         const char *featureListBuffer = NULL ) const ;
  //
  // Delete an record from the catalog.
  // CLB 24Sep: Make some assumptions about keyArg ... principally that
  //            it is at the top of the hierarchy, i.e. a parent key.
  //            I think this restricts us to two levels: parent and
  //            SubPart, but we may come back to that later.
  //            When a parent is deleted, all its SubParts go too.
  // JLH, Nov96: added another argument to specify features to touch.
  virtual int DeleteRecord (const char *keyArg,
                            const char *featureListBuffer = NULL ) const ;
  // CLB 24Sep: Delete just the SubPart record for the argument.
  //            The parent's status is unaffected.
  //
  virtual int DeleteSubPartRecord (const char * subKeyArg ) const ;
  //
  // Set the feature with information from the persistant store
  //
  virtual int GetFeature (const char *keyArg,
                          QbFeatureClass &feature) const ;
  //
  // Get the record ( raw data ) from persistant store
  //
  virtual int GetRecord( const char *keyArg,
                         const char *featureClassname,
                         QbDatumClass &data  ) const ;
  //
  // Number of objects
  //
  virtual unsigned long NumberOfRecords(const char *featureClassname) const ;
  //
  // Number of index database records
  //
  virtual unsigned long NumberOfIndexRecords(const char *featureClassname) const ;
  //
  // Number of records changes since index database was built
  //
  virtual unsigned long DeltaSinceIndexBuild(const char *featureClassName ) const ;
  //
  // Number of records changes since distance table was built
  //
  virtual
  unsigned long DeltaSinceDistTableBuild(const char *featureClassName ) const ;
  //
  // Return True iff the parent key is in the catalog.
  // N.B. Can return False if table cannot be opened.
  //
  virtual Boolean ParentKnown( const char * pKeyArg ) ;
  //
  // Return global feature informataion in datum
  //
  virtual int GetGlobalFeatureInfo( const char *featureName,
                                    QbDatumClass &datum ) const ;
  //
  // Save the global feature information in datum
  //
  virtual int PutGlobalFeatureInfo( const char *featureName,
                                    QbDatumClass &datum ) const ;
  //
  // Function to define subparts of a query object.  In Qbic this allows
  // for object ( Region of Interest) definition to be defined.
  //
  virtual int DefineSubPart( const char *parentKey, // key to base information
                             const char *subpartKey, // key for subpart
                             const QbGenericDataClass &subpart) const ;
  //
  // Return the subpart definition given the subpart key
  //
  virtual int GetSubPartDefinition( const char *subpartKey,
                                    QbGenericDataClass &subpart ) const ;
  //
  // Function to get the keys of all the subpart definitions
  //
  virtual int GetSubPartKeys( const char *parentKey,
                              unsigned int &numberOfKeys,
                              QbDatumClass &subpartKeys ) const ;
  //
  // Given a subpart key return the key to the parent
  //
  virtual int GetParentKey( const char *subpartKey,
                            QbDatumClass &parentKey ) const ;
/////////////////////////////////////////////////////////////////////////
//
// The following functions are for sampler support.  They do not
// require a catalog be opened before use.
//
/////////////////////////////////////////////////////////////////////////
  //
  // Create a new sampler table/dbm file
  //
  virtual int CreateSampler (const char *samplerName,
                             const char *featureClassName) const ;
  //
  // Drop a Sampler table/dbm file
  //
  virtual int DropSampler (const char *samplerName ) const ;
  //
  // List current sample names and their feature classes
  //
  virtual int ListSamplers (QbDatumClass &samplerList ) const ;
  //
  // Add a sample to the catalog. Feature will be computed and saved in
  // the persistent storage as a sampler in the catalog, one for each
  // sampler name in samplerList
  //
  virtual int AddSample (const  char *keyArg,
                         const char *samplerList, 
                         const QbGenericDataClass &data,
                         const QbParameterClass *parms,
                         //const int update=False
                         const UpdateType = insertRecord
                         ) const ;        // input object key
  //
  // Delete a sample from a list of samplers
  //
  virtual int DeleteSample (const char *key,
                            const char *samplerList ) const  ;
  //
  // Get a sample feature from a sampler
  //
  virtual int GetSampleFeature (const char *key,
                                const char *samplerName,
                                QbFeatureClass &feature ) const ;
  //
  // Get sampler data from a catalog
  //
  virtual int GetSampleRecord( const char *key,
                               const char *samplerName,
                               QbDatumClass &datum ) const ;
  //
  // Number of samples in a sampler
  //
  virtual unsigned long NumberOfSamples(const char *samplerName) const ;
} ;
#endif
