/*
 *  
 *  Licensed Materials - Property of IBM
 *
 *  (C) COPYRIGHT IBM Corp. 1994-1998 All rights reserved
 *
 *  US Government Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 * Filename : 69
 * Date     : 3/24/98 09:59:15
 * Version  : 1.7.3.3
 * Author   : M. Flickner and others
 * Purpose  : Define the abstract classes used for qbic extensibility
 *
 *
 */

/* End of Common Block                                               */

#ifndef __QbCreate_hpp
#define __QbCreate_hpp
#include <dmbsystm.h>
//
// To accomplish the extensibility the generic query routine need to
// be able to create specific instances of derived classes. This is
// accomplished with the createAbstractClassName functions. People
// adding their own features/indexes to the framework are required to
// modify the source to these file in QbCreate.cpp and the case for
// the specific feature. This allows the generic query routine to
// create instances of classes from a passed character string giving
// the name of the class.
//
// For each QbInterface type the user must supply the routine
// createInstance that will create an instance of the derived
// class and return a pointer to the abstract class.
//
DMBEXPHP(QbFeatureClass     *)QbCreateFeatureClass(const char *className);
DMBEXPHP(QbConnectClass     *)QbCreateConnectClass(const char *className);
DMBEXPHP(QbGenericDataClass *)QbCreateGenericDataClass(const char *className);
DMBEXPHP(QbParameterClass   *)QbCreateParameterClass(const char *className);
DMBEXPHP( QbQueryClass       *) QbCreateQueryClass(const char *className);
DMBEXPHP( const QbConnectClass *) QbGetConnection(const char *modeArg,
                                      const char *dsNameArg = NULL,
                                      const char *connectClassNameArg = NULL) ;
DMBEXPHP(int QbDestroyConnection) (const QbConnectClass *connection) ;
DMBEXPHP(const char **) QbListAllAvailableFeatures() ;
#endif
