/*
 *  
 *  Licensed Materials - Property of IBM
 *
 *  (C) COPYRIGHT IBM Corp. 1994-1998 All rights reserved
 *
 *  US Government Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 * Filename : 73
 * Date     : 7/27/98 13:52:02
 * Version  : 1.7.3.4
 * Author   : M. Flickner
 * Purpose  : Qbic Generic Data Class
 *
 *
 */

/* End of Common Block                                               */

#ifndef __QbGenDat_hpp
#define __QbGenDat_hpp
#include <dmbsystm.h>
//
// Define an abstract type for generic data.  This allows this
// framework to be used with images, speech, text, graphics....
//
class QbGenericDataClass: public QbBaseClass {
public:
    enum dataSource { unknown='u', memory='m', sample='s',
                      example='e', picker='p' } ;
private:
  QbGenericDataClass( const QbGenericDataClass& ) ; // avoid copies
  QbGenericDataClass& operator =(const QbGenericDataClass&) ;
protected:
  dataSource type ;
  QbDatumClass derivedClassData ;
  QbDatumClass exampleOrSampleFromDatabase ;
  char *catalogName ;
  char *keyName ;
  char *samplerName ;
  Boolean isQuery ;      // True if this is the query specification
  char *textDescription ;
  // Is it being used to define a SubPart
  Boolean isSubPartData ;
public:
  DMBEXPHCONSTRUCT QbGenericDataClass();
  virtual ~QbGenericDataClass() {
    QbDelete(keyName) ;
    QbDelete(catalogName) ;
    QbDelete(samplerName) ;
    QbDelete(textDescription) ;
  } ;
  // converts a generic image description into an ascii string
  virtual DMBEXPHP (int) ToQueryString(QbDatumClass &datum) ;
  // convert ascii query string filling in appropriate fields
  virtual DMBEXPHP (int) FromQueryString(const QbDatumClass &datum,
                              const char *featureClassname ) ;
  virtual DMBEXPHP (int) FromQueryString(const char *featOrSamp) ;
  const QbDatumClass &GetDerivedClassData() const
  { return derivedClassData ; }
  // This should be renamed ...
  const QbDatumClass &GetData() const
  { return exampleOrSampleFromDatabase ; }
  void SetType( const dataSource typeArg ) { type = typeArg ; }
  const dataSource GetType( ) const { return type ; }
  const char * GetKey() const { return keyName ; }
  int SetKey( const char *keyArg ) {
    QbDelete(keyName);
    keyName = QbStrDup( keyArg);
    // check that QbStrDup worked
    if ( keyName == NULL && keyArg != NULL ) return -1;
    return 0;
  }
  const char *GetCatalog() const { return catalogName ; }
  int SetCatalog( const char *catalogArg ) {
    QbDelete(catalogName) ;
    catalogName = QbStrDup( catalogArg ) ;
    // check that QbStrDup worked
    if ( catalogName == NULL && catalogArg != NULL ) return -1;
    return 0;
  }
  const char * GetSampler() const { return samplerName ; }
  int SetSampler( const char *samplerArg ) {
    QbDelete(samplerName) ;
    samplerName = QbStrDup( samplerArg ) ;
    // check that QbStrDup worked
    if ( samplerName == NULL && samplerArg != NULL ) return -1;
    return 0;
  }
  Boolean GetIsQuery() const { return isQuery ; }
  Boolean SetIsQuery(Boolean boolean) { return isQuery=boolean ; }
  int SetTextDescription( const char *text ) {
    QbDelete(textDescription) ;
    textDescription = QbStrDup( text );
    // check that QbStrDup worked
    if ( textDescription == NULL && text != NULL ) return -1;
    return 0;
  }
  char *GetTextDescription() const { return textDescription; }
  Boolean isSubPartDefinition() const { return isSubPartData ; }
  void DataIsSubPartDefinition() { isSubPartData = True ; }
  //
  // Convenience function to build a query by example object
  //
  int MakeQueryByExampleObject(const char* aCatalogName,
                                const char *aKeyName ) {
    SetType( example ) ;
    if ( SetCatalog( aCatalogName ) ) return -1; // fatal
    if ( SetKey( aKeyName ) ) return -1; // fatal
    return 0;
  }
  //
  // Convenience function to create a text/keyword query object
  //
  int MakeQueryByTextObject( const char *text ) {
    SetType( picker ) ;
    // set the keyword in the text description field of the query
    // image structure
    if ( SetTextDescription( text ) ) return -1; // fatal
    return 0;
  }
  //
  // Virtual function to encode a SubPart definition as a QbDatumClass
  // in a manner specific to the derived class (image, speech, ...)
  //
  virtual int EncodeSubPartDef( QbDatumClass & subPartDatum ) const = 0 ;
  //
  // Virtual function to decode a SubPart definition from a QbDatumClass
  // in a manner specific to the derived class (image, speech, ...)
  //
  virtual int DecodeSubPartDef( const QbDatumClass & subPartDatum ) = 0 ;
//
// From QbBaseClass
//
  virtual const QbInterfaceType IType() const
  { return GenericDataInterfaceType ; }
  virtual const char *Type() const { return "QbGenericDataClass" ; }
};
#endif
