/*
 *  
 *  Licensed Materials - Property of IBM
 *
 *  (C) COPYRIGHT IBM Corp. 1994-1998 All rights reserved
 *
 *  US Government Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 * Filename : 74
 * Date     : 7/27/98 13:53:11
 * Version  : 1.8.3.4
 * Author   : M. Flickner
 * Purpose  : This class is the implementation of QbGenericData class
 *            for image data in the QBIC system.
 *
 *
 */

/* End of Common Block                                               */

#ifndef __QbImgDat_hpp
#define __QbImgDat_hpp
#include <dmbsystm.h>
#include "QbAbsCls.hpp"
class QbGenericImageDataClass : public QbGenericDataClass {
private:
  int Cleanup() ; // frees up storage
  Impvaru *impImage, *impLut, *impMask ;
  char *filename ;  // Name of external image file
  char *maskname ;  // Name of external mask file
  //  Does this class delete the storage for image, mask & lut ?
  Boolean deleteStorage ;
  //
  // In memory image representation
  //
  unsigned char *image ;
  unsigned char *lut ;
  unsigned char *redImage ; // for future 24 bit color support
  unsigned char *greenImage ;
  unsigned char *blueImage ;
  unsigned char *mask ;
  unsigned long xsize ;
  unsigned long ysize ;
  Boolean isMaskAllOnes ;  // false if isSubPartData is true, but not conversely
public:
  //
  // Function to create a image data object from memory objects and which
  // can be passed to QbBuildQueryString.  By default the storage
  // for image, mask and lut is not deleted on reassignment or by the
  // destructor. This can be changed by passing True as the last argument.
  // The next to last argument is used to determine trigger this as a
  // subPart object.  Only the guy calling this, knows for sure.
  // (Replaces badly named MakeQueryByImageObject)
  DMBEXPHP( int ) GetImageDataFromMemory(const unsigned long width,
                                         const unsigned long height,
                                         unsigned char *imageArg,
                                         unsigned char *lutArg,
                                         unsigned char *maskArg,
                                         const Boolean subpart = False,
                                         Boolean deletesStorageArg = False) ;
  // Populate Image Data object from file (replaces MakeQueryByImageObject
  // since that didn't have anything to do with Query!)
  DMBEXPHP(  int ) ReadImageDataFromFile( const char *filename,
                                          const char *maskFilename = NULL,
                                          const Boolean subpart = False) ;
  DMBEXPHP(  int ) ReadImageDataFromPickerFile( const char *filename,
                                                const char *maskFilename = NULL,
                                                const Boolean subpart = False) ;
  // read a picker image description from file or from a string
  // (as generated by a GUI--currently supports only rectangles)
  DMBEXPHP(  int ) ReadPickerImageDescriptionFile( const char *filename,
                                             const Boolean subpart = False) ;
  DMBEXPHP(  int ) ReadPickerImageDescriptionString( const char *dstr,
                                             const Boolean subpart = False) ;
  // --------------------------------------------------------------------
  // Given the (unqualified) name of the mask file, read its contents and
  // update the mask, alwys subject to a check that the dimnsions match.
  //   NOTE: Internally relies on the QbicMaskPath environment variable.
  //
  DMBEXPHP( int ) UpdateMaskInImageObject( const char * maskFilename ) ;
  DMBEXPHCONSTRUCT QbGenericImageDataClass() : impImage(NULL), impLut(NULL),
    impMask(NULL), filename(NULL), maskname(NULL), deleteStorage(False),
    image(NULL), lut(NULL), redImage(NULL), greenImage(NULL), blueImage(NULL),
    mask(NULL), xsize(0), ysize(0), isMaskAllOnes(False) {} ;
  virtual DMBEXPHCONSTRUCT ~QbGenericImageDataClass();
  // converts private datum and type to ascii query string in returned datum
  virtual DMBEXPHP(int) ToQueryString(QbDatumClass &datum) ;
  // convert from ascii query string to internal datum and type
  virtual DMBEXPHP(int) FromQueryString(const QbDatumClass &datum,
                                        const char *featureClassname ) ;
  virtual DMBEXPHP(int) FromQueryString(const char *featOrSamp) {
       return QbGenericDataClass::FromQueryString(featOrSamp);
  }
  const char *GetFilename() const { return filename; } ;
  const char *GetMaskname() const { return maskname; } ;
  unsigned long GetXsize() const { return xsize ; } ;
  unsigned long GetYsize() const { return ysize ; } ;
  unsigned char *GetImage() const { return image ; } ;
  unsigned char *GetLut() const { return lut ; } ;
  unsigned char *GetMask() const { return mask ; } ;
  //
  // From QbGenericDataClass
  // ~~~~~~~~~~~~~~~~~~~~~~~
  //
  // Virtual function to encode a SubPart definition as a QbDatumClass
  // in a manner specific to the derived class (image, speech, ...)
  //
  virtual DMBEXPHP( int ) EncodeSubPartDef( QbDatumClass & subPartDatum ) const ;
  //
  // Virtual function to decode a SubPart definition from a QbDatumClass
  // in a manner specific to the derived class (image, speech, ...)
  //
  // NOTE:  This function assumes that this QbGenericImageDataClass has
  //        already been initialised, e.g. with ReadImageDataFromFile().
  //        This routine will then check the dimensions and set the mask.
  //
  virtual DMBEXPHP( int ) DecodeSubPartDef( const QbDatumClass & subPartDatum ) ;
  //
  // From QbBaseClass
  //
  virtual const QbInterfaceType IType() const
  { return GenericDataInterfaceType ; }
  virtual const char *Type() const { return "QbGenericImageDataClass" ; }
} ;
#endif
