/*
 *  
 *  Licensed Materials - Property of IBM
 *
 *  (C) COPYRIGHT IBM Corp. 1994-1998 All rights reserved
 *
 *  US Government Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 * Filename : 79
 * Date     : 10/27/98 15:06:52
 * Version  : 1.7.4.3
 * Author   : M. Flickner
 * Purpose  : QBIC server query evaluator class
 *
 *
 */

/* End of Common Block                                               */

#ifndef __QbQueryS_hpp
#define __QbQueryS_hpp
#include <dmbsystm.h>
#include "QbAbsCls.hpp"
#include "QbCatCls.hpp"
class QbQueryServerClass : public QbQueryClass {
private:
  int remote;
  enum {maxNumberOfFeatures=16} ;
  unsigned long queryResultsRequested ; // how many items do we return
  int results_free;
  int numberObjectFeatures ;
  int numberSceneFeatures;
  int numGroupScene, numberGroupSceneFeatures[maxNumberOfFeatures];
  int numGroupObject, numberGroupObjectFeatures[maxNumberOfFeatures];
  char catalogName[256] ;
  QbCatalogClass  *catalog ;
  const QbConnectClass *connect ;
  QbDatabaseClass *dbSceneFeatures[maxNumberOfFeatures] ;
  QbDatabaseClass *dbSceneGroupFeatures[maxNumberOfFeatures][maxNumberOfFeatures] ;
  QbKeyDatabaseClass *sceneRestrictionList ;
  QbFeatureClass *sceneFeatures[maxNumberOfFeatures] ;
  QbFeatureClass *sceneGroupFeatures[maxNumberOfFeatures][maxNumberOfFeatures] ;
  QbGenericDataClass *sceneGenericData[maxNumberOfFeatures] ;
  QbParameterClass *sceneParameters[maxNumberOfFeatures] ;
  QbParameterClass *sceneGroupParameters[maxNumberOfFeatures][maxNumberOfFeatures] ;
  float sceneFilterCost[maxNumberOfFeatures] ;
  float sceneGroupFilterCost[maxNumberOfFeatures][maxNumberOfFeatures] ;
  float sceneDistanceCost[maxNumberOfFeatures] ;
  float sceneGroupDistanceCost[maxNumberOfFeatures][maxNumberOfFeatures] ;
  QbDatabaseClass *dbObjectFeatures[maxNumberOfFeatures] ;
  QbDatabaseClass *dbObjectGroupFeatures[maxNumberOfFeatures][maxNumberOfFeatures] ;
  QbKeyDatabaseClass *objectRestrictionList ;
  QbFeatureClass *objectFeatures[maxNumberOfFeatures] ;
  QbFeatureClass *objectGroupFeatures[maxNumberOfFeatures][maxNumberOfFeatures] ;
  QbGenericDataClass *objectGenericData[maxNumberOfFeatures] ;
  QbParameterClass *objectParameters[maxNumberOfFeatures] ;
  QbParameterClass *objectGroupParameters[maxNumberOfFeatures][maxNumberOfFeatures] ;
  float objectFilterCost[maxNumberOfFeatures] ;
  float objectGroupFilterCost[maxNumberOfFeatures][maxNumberOfFeatures] ;
  float objectDistanceCost[maxNumberOfFeatures] ;
  float objectGroupDistanceCost[maxNumberOfFeatures][maxNumberOfFeatures] ;
  QbFeatureClass *evaluateDistanceFeature ;
  const char *queryKey ; // used for fast (precomputed) single feature queries
  int SceneGroupPrepare();
  int SceneGroupExecute() ;
  int ObjectGroupPrepare();
  int ObjectGroupExecute() ;
  int idxScene[maxNumberOfFeatures];
  int idxObject[maxNumberOfFeatures];
  int SceneNoGroupPrepare() ;
  int ObjectNoGroupPrepare() ;
  int Create( const char *query ) ; // create the query
  int Prepare() ;  // prepare and/or optimize the query
  int Execute() ;  // execute/run the query
  QbKeyDatabaseClass * RunQuery(void); 
  
  int Cleanup() ;      // cleanup storage for possible reuse of class
  // CLB 26 Nov 96 - Make the following three functions generic in the
  //                 sense that they operate on either scene or object
  //                 (i.e. "area" - until I think of something better)
  //
  int KeySort( int                 numberAreaFeatures
              , QbFeatureClass   ** areaFeatures
              , QbParameterClass ** areaParameters
              ) ;
  void GroupSort( int                  numberGroup
                , int                * idx
                , QbParameterClass *** groupParameters[maxNumberOfFeatures][maxNumberOfFeatures]) ;
  // Execute query on single feature
  int SingleFeature( QbDatabaseClass    * dbAreaFeatures
                   , QbKeyDatabaseClass * areaRestrictionList
                   , QbFeatureClass     * areaFeatures
                   , QbParameterClass   * areaParameters
                   ) ;
  // Execute query on multiple features
  int MultiFeatures( int                   numberAreaFeatures
                   , QbDatabaseClass    ** dbAreaFeatures
                   , QbKeyDatabaseClass  * areaRestrictionList
                   , QbFeatureClass     ** areaFeatures
                   , QbParameterClass   ** areaParameters
                   ) ;
  // Execute query on multiple features grouped for multipass
  // CLB 27 Nov 96 - function for handling mixed scene/object queries
  // This routine starts with objects then merges scene matches.
  int MixedFeatures() ;
//
// This is a OQS style query evaluator
//
// Query specifications can be created using the function QbBuildQueryString.
// The result of a query is a QbKeyDatabaseClass.
//
public:
  virtual QbKeyDatabaseClass * Evaluate(const char *querySpecification);
  virtual QbKeyDatabaseClass * Evaluate( 
                              const int numberOfResults,
                              QbCatalogClass *cat,
                              QbKeyDatabaseClass *restrictionList,
                              QbKeyDatabaseClass *partOfRestrictionList,
                              int numberOfFeatures,
                              QbGenericDataClass **data,
                              QbFeatureClass   **feats,
                              QbParameterClass **parms);

  DMBEXPHCONSTRUCT QbQueryServerClass( const QbConnectClass *inQbicConnection = NULL ) ;
  virtual DMBEXPHCONSTRUCT ~QbQueryServerClass() ;
  // from QbBaseClass
  virtual const QbInterfaceType IType() const
  { return QueryInterfaceType ; }
  virtual const char *Type() const { return "QbQueryServerClass" ; }
  //
  // The function below is needed to make UDF distance evaluation more
  // efficient.  This function can only be call if a query is first
  // evaluated - only works with single feature queryies ....
  //
  virtual int EvaluateDistance( const char *key,
                                const float abortDistance,
                                float &distance ) ;
};
//
// Utility function to make a query string
//
DMBEXPHP( int)
QbBuildQueryString( const int numberOfResults,
                    const char *catalogName,
                    QbKeyDatabaseClass *restrictionList,
                    QbKeyDatabaseClass *partOfRestrictionList,
                    const int numberOfFeatures,
                    const char **featureClassnames,
                    QbParameterClass **parms,
                    QbGenericDataClass **data,
                    QbDatumClass &queryString ) ;
#endif
