/*
 *  
 *  Licensed Materials - Property of IBM
 *
 *  (C) COPYRIGHT IBM Corp. 1994-1998 All rights reserved
 *
 *  US Government Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 * Filename : 65
 * Date     : 3/24/98 09:59:39
 * Version  : 1.3.3.3
 * Author   : Myron Flickner
 * Purpose  : Simple string class
 *
 *
 */

/* End of Common Block                                               */

#ifndef __QbStringClass_hpp
#define __QbStringClass_hpp
#include <string.h>
#include <ctype.h>
#include <dmbsystm.h>
#include "quError.h"
#include "Qbic.h"  // for Boolean
class QbStringClass {
private:
  unsigned long allocatedSize ;
  unsigned long currentSize ; // size includes the '\0'
  char *buffer ;
  Boolean destructorDelete ;
 
  QbStringClass( const QbStringClass& ) ; // no copies
  DMBEXPHP (void) IncreaseSpace(unsigned long spaceWanted ) ;
protected:
  Boolean IamOK;
 
public:
  Boolean IsOK() { return IamOK; }
 
  // Simple constructor from a string
 
  DMBEXPHCONSTRUCT QbStringClass( const char *string,
                                  Boolean destructorDeleteArg = True ) ;
  // Constructor to specify the size of the buffer
 
  DMBEXPHCONSTRUCT QbStringClass( const unsigned long size,
                                  Boolean destructorDeleteArg = True ) ;
 
  // destructor
 
  DMBEXPHCONSTRUCT ~QbStringClass() ;
  // allow for a cast to a unsigned char and char
  // Return NULL if size is zero or 1 (only '\0' byte)
 
  operator unsigned char * () const {
    if (currentSize>1) return (unsigned char *) buffer ;
    else return NULL ;
  }
  operator char * () const {
    if (currentSize>1) return buffer ;
    else return NULL ;
  }
  operator const char *() const {
    if (currentSize>1) return buffer ;
    else return NULL ;
  }
  QbStringClass& operator = (const char *in) {
    int length ;
    if ( in ) length = strlen((char *) in ) + 1 ; /* for HP */
    else length = 1;
    IncreaseSpace( length ) ;
    if(allocatedSize >= length) {
      if ( in ) memcpy( buffer, in, length ) ;
      else buffer[0] = '\0';
      currentSize = length ;
    }
    return *this ;
  }
 
  DMBEXPHP(unsigned long) GetSize() const { return currentSize ; }

  DMBEXPHP(void) SetDestructorDelete(Boolean destructorDeleteArg) {
    destructorDelete = destructorDeleteArg;
  }
  // concatenate to the end of the string
 
  QbStringClass& operator += ( const char *str )
  {
    unsigned long length, spaceNeeded ;
    if (!str) return *this; // nothing to do
    // from here on in, str != NULL
    length = strlen( (char *)str ) ;  // no +1 since we're appending
    // handle case when currentSize = 0 ;
    spaceNeeded = (currentSize ? currentSize : 1 ) + length ;
    IncreaseSpace(spaceNeeded) ;
    if(allocatedSize >= spaceNeeded) {
      if( 0 == currentSize )
        memcpy( &(buffer[currentSize]), str, length + 1 ) ;
      else
        memcpy( &(buffer[currentSize-1]), str, length + 1 ) ;
 
      currentSize = spaceNeeded ;
    }
    return *this ;
  }
 
  //
  // Encode a binary buffer and concat it to the string. For now we hex
  // encode - evenutally we need to mime encode and possible lzw compress
  // and mime encode
  //
  int EncodeAndConcat( const unsigned char *input, const unsigned long size)
  {
    unsigned long i,j, spaceNeeded ;
    char *output ;
    static const char hexTable[] = "0123456789abcdef" ;
 
    spaceNeeded = (currentSize ? currentSize : 1 ) + 2*size ;
    IncreaseSpace( spaceNeeded ) ;
    if(allocatedSize >= spaceNeeded) {
      if( 0 == currentSize ) output = &(buffer[currentSize]) ;
      else output = &(buffer[currentSize-1]) ;
 
      for(j=i=0; i<size; i++) {
        output[j++] = hexTable[ input[i] >> 4 ] ;
        output[j++] = hexTable[ input[i] & 0x0f ] ;
      }
      output[j] = '\0' ; // null terminate
 
      currentSize = spaceNeeded ;
      return 0 ;
    }
    else return -1;
  }
} ;
static inline int hexToInt( const char c )
{
  switch ( tolower(c) ) {
  case '0' : return 0 ;
  case '1' : return 1 ;
  case '2' : return 2 ;
  case '3' : return 3 ;
  case '4' : return 4 ;
  case '5' : return 5 ;
  case '6' : return 6 ;
  case '7' : return 7 ;
  case '8' : return 8 ;
  case '9' : return 9 ;
  case 'a' : return 10 ;
  case 'b' : return 11 ;
  case 'c' : return 12 ;
  case 'd' : return 13 ;
  case 'e' : return 14 ;
  case 'f' : return 15 ;
  default: return 0 ;
  }
}
// function to reverse EncodeAndConcat
static inline int DecodeBuffer( const char *encoded, unsigned char *decode,
                                unsigned long decodeSpace )
{
  unsigned long i,j;
  for(j=i=0; isxdigit( encoded[i] ); i+=2 ) {
    decode[j++] = (hexToInt(encoded[i]) << 4 ) | hexToInt( encoded[i+1] ) ;
    if( j > decodeSpace ) {
      QbError(QERR_INFO, QERR_NON_FATAL,
              "QbString:DecodeBuffer found more data than requested");
      return -1 ; // need QbError posted here
    }
  }
  return 0 ;
}
#endif
