/*
 *  
 *  Licensed Materials - Property of IBM
 *
 *  (C) COPYRIGHT IBM Corp. 1994-1998 All rights reserved
 *
 *  US Government Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 * Filename : 62
 * Date     : 6/18/98 19:24:38
 * Version  : 1.9.3.6
 * Author   : M. Flickner
 * Purpose  : Useful utilites for QBIC
 *
 *
 */

/* End of Common Block                                               */

#ifndef __QbUtil_h
#define __QbUtil_h
#if __STDC__ || defined(__cplusplus)
#define P_(s) s
#else
#define P_(s) ()
#endif
#include <dmbsystm.h>
/* QbUtil.c */
extern char *QbBuildFilename P_((
	const char *filename,
	const char *ext
	));
extern char *QbBuildFilenameWithEVPath P_((
	const char *filename,
	const char *ext,
        const char *environmentVariableGivingPath
	));
extern char *BuildMaskFilename P_((
	const QbSceneType *scene,
	const int i
	));
extern char *QbStripPath P_((
	char *filenameWithPath
	));
DMBEXPHP( char *) QbStrCat P_(( const char *const str1,
                           const char *const str2,
                           const char *const str3 = NULL ,
                           const char *const str4 = NULL,
                           const char *const str5 = NULL ,
                           const char *const str6 = NULL )) ;
DMBEXPHP(char *) QbStrDup P_(( const char *string,
                           unsigned long extra = 0 )) ;
extern Impvaru * QbCreateColorLut() ;
extern const unsigned char *QbGetColorLut() ;
extern Impvaru *QbErrorDiffuseImage P_((
	Impvaru *image,
	const char *lut
	));
extern int QbGetNumberOfColors P_(( const char color = '\0' )) ;
/*
 * Routine to create a good quality palletized thumbnail for qbic
 * using the Qbic colormap.
 */
extern int QbBuildImageThumbnail P_(( const unsigned char *const image,
                                      const unsigned int imageWidth,
                                      const unsigned int imageHeight,
                                      const unsigned char *const imageLut,
                                      unsigned char *const thumb,
                                      const unsigned int thumbWidth,
                                      const unsigned int thumbHeight )) ;
// compute normalized weighted euclidian distance
extern fastfloat NormWtdEucDistSqr P_((
	fastfloat array1[],
	fastfloat array2[],
	fastfloat weights[],
	int n
	));
extern DMBEXPHP (const char *)   QbNextWord P_(( const char *str) ) ;
extern DMBEXPHP (unsigned long)  QbNextWordSize P_( ( const char *str )) ;
extern DMBEXPHP (Boolean)        QbIsBigEndian( void ) ;
extern DMBEXPHP (unsigned short) QbSwabShort( const unsigned short s ) ;
extern DMBEXPHP (unsigned long)  QbSwabLong( const unsigned long l ) ;
extern DMBEXPHP (unsigned short) QbShortToBigEndian( const unsigned short s ) ;
extern DMBEXPHP (unsigned long)  QbLongToBigEndian( const unsigned long l ) ;
//
// write out a floating point image - rescaling if needed
//
extern int QbWriteImage(const float *image,
                        const unsigned long xSize,
                        const unsigned long ySize,
                        const char *filename,
                        const char *prefix = NULL,
                        const Boolean setScale = False,
                        const float minArg = 0.0,
                        const float scaleArg = 1.0 ) ;
//
// read an image and write the thumbnail (called by QbMkThmb so we
// don't need to export anything in this method except this one thing)
//
extern DMBEXPHP(int) QbReadImageWriteThumbnail( char *imagename,
                                                char *thumbname,
                                                int thumbWidth,
                                                int thumbHeight,
                                                Boolean pad=True) ;
 
//
// read an image and write a 24bit thumbnail (called by QbMkThmb so we
// don't need to export anything in this method except this one thing)
//
extern DMBEXPHP(int) QbReadImageWrite24bitThumbnail( char *imagename,
						     char *thumbname,
						     int thumbWidth,
						     int thumbHeight,
						     Boolean pad=True) ;
#ifdef SCENE_OBJECTS
extern QbSceneType *QbMallocScene P_((
	const unsigned int nobjs
	));
extern QbSceneType *QbReallocScene P_((
	QbSceneType *scene,
	const unsigned int nobjs
	));
extern void QbFreeSceneImages P_((
	QbSceneType *scene,
	const Boolean ZoomedOnly
	));
extern void QbFreeScene P_((
	QbSceneType *scene
	));
extern QbSceneType *ReadSceneAndObjects P_((
	const char *fn
	));
extern void QbWriteChangedObjects P_((
	const QbSceneType *scene
	));
extern void QbWriteSceneTextInfo P_((
	const char *filename,
	const char *SceneText
	));
#endif
#undef P_
#endif
