/*
 *  Licensed Materials - Property of IBM
 *
 *  (C) COPYRIGHT IBM Corp. 1994-1998 All rights reserved
 *
 *  US Government Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 * Filename : 14
 * Date     : 10/19/98 15:27:13
 * Version  : 1.4
 * Author   : Xiaoming Zhu (based on the work by Flick and Jim Hafner)
 * Purpose  : A high level Wrapper class for commandline programs such as
 *            QbMkDbs, QbQBE, QbMkThmb, QbDumpDb etc.
 *
 */

/* End of Common Block                                               */

#ifndef __QBICWRAP_HPP
#define __QBICWRAP_HPP

#include "QbAbsCls.hpp"
#include "QbImgDat.hpp"
#include "QbCatCls.hpp"
#include "QbCreate.hpp"
#include "QbQueryS.hpp"
#include "quError.h"
#include "QbEnv.h"
#include "QbInp.h"

typedef Boolean QbBoolean ;

#ifdef __MWERKS__
static char *getHeader(char *input, char *output, int *ict);
static int Unix2MacDir(char *input, char *output);
static int Unix2MacKey(char *input, char *output);
#endif

class QbicWrapClass {

private:

  const QbConnectClass          *connection ;
  QbCatalogClass                *catalog;
  QbFeatureClass                *featureClass[NUMBER_OF_FEATURES];
  QbParameterClass              *parmsClass[NUMBER_OF_FEATURES];
  QbFeatureClass                *sampleFeatureClass[NUMBER_OF_FEATURES];
  QbParameterClass              *sampParmsClass[NUMBER_OF_FEATURES];
  QbGenericImageDataClass       *queryImageOrKey[NUMBER_OF_FEATURES];
  QbQueryServerClass            *queryClass;

  int                            numberOfFeatures, numberOfSamplers;
  int                            nHits;
  int                            thumb_x, thumb_y;
  QbBoolean                      thumb_24color, thumb_pad;
  char                           *imageListName;
  int                            catNeedOpen, featNeedAdd;
  char                           catalogName[256], databaseName[256], mode[2];
  char                           featNames[NUMBER_OF_FEATURES][200];
  char                           sampNames[NUMBER_OF_FEATURES][200];
  char                           sampFeatNames[NUMBER_OF_FEATURES][200];
  char                           *updateStr;
  char                           keyWord[128];
  QbDatumClass                   samplerList ;
  QbGenericImageDataClass        image;
  QbBoolean                      remove, erase, subparts, OK ;
  QbCatalogClass::UpdateType     update;
  int                            SP;
  int                            cmdArgc;
  char                           **cmdArgv;

  char *parseParentKey           (char *imgNames);
  char *parseNextChild           (char *imgNames);

  int  QbicWrapImageObj          ();
  int  QbicWrapCDF               (int *argc, char ***argv, char *mode="r");

  int  QbicWrapInit              ()
     {
       for(int ict=0; ict<NUMBER_OF_FEATURES; ict++) queryImageOrKey[ict]=NULL;
       numberOfFeatures = numberOfSamplers = 0;
       imageListName = NULL;
       catNeedOpen = featNeedAdd = 0;
       strcpy(catalogName, "ibm33");
#ifndef __MWERKS__
       strcpy(databaseName, "QbicData");
#else
       strcpy(databaseName, ":QbicData");
#endif
       strcpy(mode, "r");
       update = QbCatalogClass::insertRecord;
       updateStr = "Insert";
       remove = erase = subparts = OK = False;
       catalog = NULL;
       connection = NULL;
       nHits = 20;
       thumb_x = thumb_y = 100;
       thumb_24color = thumb_pad = False;
       keyWord[0]='\0';
       SP = 0;
       return 0;
     }

     int ProcessInputFileLine        (char *inputFileLine, QbBoolean remove);
     QbBoolean isSubPart             (char *cmdInput);
     int OpenCatalog                 (char *catName);
     int OpenCatalog                 (void) {return OpenCatalog(catalogName);}
     int CreateCatalog               (char *catName);
     int CreateCatalog               (void) {return CreateCatalog(catalogName);}
     int ConnectToDatabase           (void);
     int ConnectToDatabase           (char *dirName, char *mode);

public:

  QbicWrapClass                   ();
  QbicWrapClass                   (char *catName, char *dirName, char *mode);

  int QbicWrapQBE                 (int argc, char **argv, QbStringClass
                                                                   *str=NULL);
  int QbicWrapDBS                 (int argc, char **argv, QbStringClass 
                                                                   *str=NULL);
  int QbicWrapThm                 (int argc, char **argv, QbStringClass 
                                                                   *str=NULL);
  int QbicWrapDumpDb              (int argc, char **argv, FILE *stream=stdout);

  char *QbicWrapGetKeyWord        (void) {return keyWord;}
  void QbicWrapSetKeyWord         (char *kw) {strcpy(keyWord, kw);}

  void SetScreenPrint             (int sp) {SP = sp;}
  int QbicWrapSetReturnedKeys     (int keys) {int tmp = nHits;
                                                  nHits = keys;
                                                  return tmp;
                                             }

  void QbicWrapThumbXY            (int xlen, int ylen) {
                                                  thumb_x = xlen;
                                                  thumb_y = ylen;
                                  }

  void QbicWrapSetThumb24Color    (QbBoolean color) {thumb_24color = color;}
  void QbicWrapSetPad             (QbBoolean pad) {thumb_pad = pad;}

  int QbicWrapThumb               (char *inImage, char *outImage) {
                                     if(thumb_24color) {
                                        return QbReadImageWrite24bitThumbnail(
                                                            inImage,  
                                                           outImage,  
                                                            thumb_x, 
                                                            thumb_y, 
                                                         thumb_pad);
                                     } else {
                                        return QbReadImageWriteThumbnail(
                                                            inImage,  
                                                           outImage,  
                                                            thumb_x, 
                                                            thumb_y, 
                                                         thumb_pad);
                                     }
                                  }

  int ListCatRecords              (char *FeatureName) {
                                     if(!catalog) return -1;
                                     return (int) 
                                        catalog->NumberOfRecords(FeatureName);
                                  }

  int QbicWrapClassConnect        (void);
  int QbicWrapClassConnect        (char *catName, char *dirName, char *mode);

  int InsertFeatures              (char *featureName);
  int DeleteFeatures              (char *featureName);

  int QbicWrapInsertImage         (char *imgNames);
  int QbicWrapDeleteImage         (char *imgNames);

  int QbicWrapInsertSubImage      (char *imgNames);
  int QbicWrapDeleteSubImage      (char *imgNames);

  int QbicWrapQueryDB             (QbStringClass *str);
  int QbicWrapRandomQueryDB       (QbStringClass *str, QbBoolean ranType=True);

  int QbicWrapQueryImage          (char *imgNames, char *maskName);
  int QbicWrapQueryKey            (char *imgKeys);
  int QbicWrapQueryString         (char *imgKeys);
  int QbicWrapQueryPicker         (char *imgKeys, char *maskName);

  int QbicWrapCleanUp             (void);

  char *ListAllFeatures           (void);
  char *ListCatFeatures           (void);

  QbParameterClass *GetParamForFeature (char *featureName);

  int GetParameters               (QbStringClass *str);

  int GetFeatureDataKey           (char *imgName, QbStringClass *str);

  int GetFeatureDataImage         (char *imgName, char *maskName,
                                         QbStringClass *str);

  int GetFeatureDataPicker        (char *imgName, char *maskName,
                                         QbStringClass *str);

  int GetFeatureDataString        (char *imgName, QbStringClass *str);

  ~QbicWrapClass();
};

static int QbMkThmbUsage(FILE *stream, QbStringClass *str=NULL)
{
  char usage1 [] = "QbMkThmb [options] imageName thumbnailName\n";
  char usage2 [] = "QbMkThmb -l imageFile [options]\n";
  char options [] = 
    "Where options are\n"
    "\t-p             pad boundary with black to fill requested size\n"
    "\t-n             don't pad boundary, adjust dimensions to scale\n"
    "\t-c             use fixed colormap (default)\n"
    "\t-t             use 24bit true color, not the colormap\n"
    "\t-w width       requested thumbnail width\n"
    "\t-h height      requested thumbnail height\n"
    "\t-o directory   output directory for thumbnail (with -l option only)\n"
    "\t-s suffix      thumbnail suffix added to image name, default is\n"
    "\t               \".thumb0.gif\" (with -l option only)\n"
    "\t-e EnvVar=EnvValue  used to set general Qbic environment variable\n"
    "\te.g. QbMkThmb -t -w 128 -h 128 flower01.jpg flower01.jpg.thumb0.gif\n";
  if(NULL == str) {
     if(stream) fprintf(stream, "\nUsage:\n\n%s \nOR\n%s\n%s", 
                        usage1, usage2, options);
     else       fprintf(stderr, "\nUsage:\n\n%s \nOr\n%s\n%s", 
                        usage1, usage2, options);
  }
  else {
     *str += usage1;
     *str += "OR\n";
     *str += usage2;
     *str += options;
  }
  return 0;
}

static int QbMkDbsUsage(FILE *stream, QbStringClass *str=NULL)
{
/*
    "\t-s sampler:feature[:parms]\n"
    "\t             adds (removes if -r specified) the sampler to the catalog\n"
    "\t             using the (optional) parameters for feature computation\n"
*/
  const char usage[] =
    "[-r | -i | -u | -U ] [-E] [-c catalog]\n"
    "[-d database]\n"
    "[-f feature1[:parms1] [-f feature2[:parms2]] ...] \n"
    "[-s sampler1:feature1[:parms1] [ -f sampler2:feature2[:parms2]] ...]\n"
    "[-e EnvVar=EnvValue ...] \n"
    "[-l imageFileName |\n"
    "   image1 [ image2 ...]\n"
    "\t-r           removes the record or sample from the catalog or sampler\n"
    "\t-i           incremental update record: add data in new features classes\n"
    "\t-u           update record: replace/refresh data in all feature classes\n"
    "\t-U           update record: replace/refresh/insert data in all feature\n"
    "\t             classes\n"
    "\t-c catalog   specifies the catalog name (defaults to \"test\")\n"
    "\t-d database  specifies the database name (defaults to \"QbicData\"\n"
    "\t-E           erases the catalog (or listed features from the catalog)\n"
    "\t             or sampler (use with caution!)\n"
    "\t-f feature[:parms]\n"
    "\t             adds (removes if -E specified) the feature to the catalog\n"
    "\t             using the (optional) parameters for feature computation\n"
    "\t-e EnvVar=EnvValue\n"
    "\t             to set general environment variable\n"
    "\t -l imageFileName\n"
    "\t             name of file with one image+masks specification per line\n"
    "\t             (in same syntax as what follows the -p option below)\n"
    "\t image1 [image2]\n"
    "\t             list of images to add or remove\n"
    "\t NOTE: -l option not available with commandline list of image/masks.\n";
  if(NULL == str) {
     if(stream) fprintf(stream, "Usage:\n%s", usage);
     else       fprintf(stderr, "Usage:\n%s", usage);
  }
  else {
     *str += "Usage:\n";
     *str += usage;
  }
  return 0 ;
}

static int QbQBEUsage(FILE *stream, QbStringClass *str=NULL)
{
  const char usage1[] =
    "-Q querySpec\n"
    "\t-Q querySpec specifies a file to read for the query specification\n";
  const char usage2[] =
    "-h | -l | -L | -P | -r | -i|I|G|D query-image \n"
    "\t [-c catalog] [-d database] [-T keyword]\n"
    "\t [-f|F feature1[:parms1] [-f|F feature2[:parms2] ...]]\n"
    "\t [-n nresults] [-t] [-e EnvVar=EnvValue ]\n"
    "\t-h           help -- dump this usage message\n"
    "\t-l           the list of feature classes in the catalog (-c or default)\n"
    "\t-L           the list of feature classes known to Qbic\n"
    "\t-P           dump the default parameters for each of the -F features\n"
    "\t-r           make a \"random\" query -- returns a list of random keys\n"
    "\t-R           make a \"random\" query -- returns the same set random keys\n"
    "\t             database keys in the feature (-f), catalog (-c or default)\n"
    "\t             and database (-d or default)\n"
    "\t-i dbkey     specify the database key (key must exist in db)\n"
    "\t             for the query\n"
    "\t-I image     specify the image file for the query\n"
    "\t-S description  specify the GUI-generated query image description string\n"
    "\t-c catalog   specifies the catalog name (defaults to \"test\")\n"
    "\t-d database  specifies the database name (defaults to \"QbicData\")\n"
    "\t-T keyword   specifies the single restriction-keyword to filter with\n"
    "\t             (first query)\n"
    "\t-f feature:parms\n"
    "\t             specifies the feature class(es) to use in the main (second)\n"
    "\t             query together with (optional) weights are parameters for\n"
    "\t             that feature -- see documentation for details on parameters\n"
    "\t-F feature   specifies the feature data to dump for -i,-I,-G,-D,-P options\n"
    "\t-n #         specifies the maximum number of results to return\n"
    "\t             (from second query) \n"
    "\t-e EnvVar=EnvValue\n"
    "\t             to set general Qbic environment variables\n"
    "\t-t           time the main query in milliseconds\n";
  if(NULL == str) {
     if(stream) fprintf(stream, "\nQbQBE %s OR \nQbQBE %s", usage1, usage2);
     else       fprintf(stderr, "\nQbQBE %s OR \nQbQBE %s", usage1, usage2);
  }
  else {
     *str += usage1;
     *str += " OR ";
     *str += usage2;
  }
  return 0;
}

static int QbDumpDbUsage(FILE *stream, QbStringClass *str = NULL)
{
  const char DumpUsage[] =
    "Usage:\t QbDumpDb -d database dbmFileName [-f featureName] \n"
    "OR:   \t QbDumpDb -c catalog -d database -f feature1 [-f feature2 ...]\n";
  if(NULL == str) {
     if(stream) fprintf(stream, "\n%s", DumpUsage);
     else       fprintf(stderr, "\n%s", DumpUsage);
  }
  else *str = DumpUsage;
  return 0;
}

#ifdef __MWERKS__
static char *getHeader(char *input, char *output, int *ict)
/* ------------------------------------------------------------------
   do the following mapping at the beginning of the input string:

   ./ => :
  ../ => ::
-------------------------------------------------------------------*/
{
char *p = input, *q;
   if( *p != '.') return input;
   output[*ict] = ':';
   p++; (*ict)++;
   if( *p != '.') return p+1;
   output[*ict] = ':';
   p++; (*ict)++;
   if( *p != '.') return p+1;
   else return NULL;
}

static int Unix2MacDir(char *input, char *output)
{
int  ict = 0, len = strlen(input);
char *p=input;

   p = getHeader(p, output, &ict);
   while(*p == '.') {
     p = getHeader(p, output, &ict);
     ict--;
   }
   if(ict == 0) {
     if(!strchr(p, '/') && !strchr(p, ':')) output[ict++] = ':';
     else if (*p == '/') {
       p++;
     } else {
       if(strchr(p, '/')) output[ict++] = ':';
     }
   }
   while(*p != '\0') {
     if(*p == '/') output[ict++] = ':';
     else          output[ict++] = *p;
     p++;
   }
   output[ict] = '\0';
   return 0;
}

static int Unix2MacKey(char *input, char *output)
{
int  ict = 0, len = strlen(input);
char *p=input;

   if(*p == '/') p++;
   while(*p != '\0') {
     if(*p == '/') output[ict++] = ':';
     else          output[ict++] = *p;
     p++;
   }
   output[ict] = '\0';
   return 0;
}
#endif  // __MWERK

#endif // __QBICWRAP
