/*
 *  
 *  Licensed Materials - Property of IBM
 *
 *  (C) COPYRIGHT IBM Corp. 1994-1998 All rights reserved
 *
 *  US Government Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 * Filename : 95
 * Date     : 11/5/98 18:13:46
 * Version  : 1.4.4.3
 * Author   : Ronald J. Barber
 * Purpose  : Error Generator Routine Header for QBIC
 *
 *
 */

/* End of Common Block                                               */

#ifndef __quError_h
#define __quError_h
#define QERR_BASE 10000
#ifndef __MWERKS__
#if defined(__MVS__)
#include <stdlib.h>
#else
#include <malloc.h>
#endif
#endif
#include <stdio.h>
#include <dmbsystm.h>
/* some macros I may use -- not final yet  */
#define QB_ERROR_RETURN(rc)  if ( (rc) ) return (rc);
#define QB_RETURN_IF_ERROR(rc)  if ( (rc) ) return (rc);
#define QB_RETURN_MINUS_1    return(-1);
typedef enum {
  QERR_NOT_FOUND               = QERR_BASE,
  QERR_INFO,
  QERR_NONE,
  QERR_OBJ_NULL,
  QERR_DATUM_NULL,
  QERR_NEW_FAIL,
  QERR_MEM,
  QERR_MEM_WHERE,
  QERR_SIZE_MISMATCH,
  QERR_IMAGE_TOOSMALL,
  QERR_ANYREAD_NOFILE,
  QERR_ANYREAD_SIZE_MISMATCH,
  QERR_BAD_COERCE,
  QERR_BAD_COERCE2,
  QERR_BAD_DATATYPE,
  QERR_FILE,
  QERR_FILE_NULL,
  QERR_FILE_FORMAT,
  QERR_FILE_OVERRUN,
  QERR_OBJ_NAME,
  QERR_CLR_SQRT,
  QERR_CLR_INVALID,
  QERR_CLR_METHOD,
  QERR_DIST_MAX,
  QERR_CLST_SZ,
  QERR_CLST_MAP,
  QERR_DB_CLASS,
  QERR_DB_FUNC,
  QERR_ANY_ERROR,
  QERR_CONNECT_UNKNOWN,
  QERR_CONNECT_FAIL,
  QERR_DATA_UNKNOWN,
  QERR_QUERY_UNKNOWN,
  QERR_CAT_OPEN_NULL,
  QERR_CAT_OPEN_FAIL,
  QERR_CAT_INSERT_FAIL,
  QERR_CAT_UPDATE_FAIL,
  QERR_CAT_UNKNOWN_FEAT,
  QERR_ADDRECORD_PRECHECK_FAIL,
  QERR_ADDRECORD_FAIL,
  QERR_ADDRECORD_DIST_FAIL,
  QERR_ADDRECORD_INDEX_FAIL,
  QERR_SUBCAT_OPEN_FAIL,
  QERR_FEAT_UNKNOWN,
  QERR_FEAT_CREATE_FAIL,
  QERR_FEAT_DELETE_FAIL,
  QERR_FEAT_OPEN_FAIL,
  QERR_FEAT_RETRIEVE_FAIL,
  QERR_FEAT_UPDATE_FAIL,
  QERR_FEAT_INSERT_FAIL,
  QERR_FEAT_REMOVE_FAIL,
  QERR_FEAT_CREATET_FAIL,
  QERR_SAMP_OPEN_FAIL,
  QERR_SAMP_RETRIEVE_FAIL,
  QERR_SAMP_INSERT_FAIL,
  QERR_SAMP_UPDATE_FAIL,
  QERR_SAMP_DELETE_FAIL,
 
  QERR_IDX_OPEN_FAIL,
  QERR_IDX_CREATET_FAIL,
  QERR_IDX_REMOVE_FAIL,
  QERR_INDEX_OPEN_FAIL,
  QERR_INDEX_UPDATE_FAIL,
  QERR_INDEX_INSERT_FAIL,
  QERR_INDEX_RETRIEVE_FAIL,
  QERR_INDEXINCR_UPDATE_FAIL,
  QERR_INDEXINCR_RETRIEVE_FAIL,
  QERR_DISTINCR_UPDATE_FAIL,
  QERR_DISTINCR_RETRIEVE_FAIL,
  QERR_GINFO_FAIL,
  QERR_SHAPE_AXIS,
  QERR_SHAPE_TQL1,
  QERR_IO_WRONGNUM,
  QERR_LAST
  } QERR_Type;
typedef enum {
  QERR_NON_FATAL = 0,
  QERR_FATAL     = 1
} QERR_SeverityType;
typedef struct {
  QERR_Type           message_number;
  QERR_SeverityType   message_severity;
  char                message[4096];
} DMBEXPHCONSTRUCT QbErrorStructureType;
extern DMBEXPHP (void) QbInitializeErrorStructure();
extern int  QbQuiet;  /* Quiet Flag (Shuts off printf's) */
 
extern void *QbMalloc(size_t size, QERR_Type msgnum,
		      QERR_SeverityType Sev);
extern void *QbCalloc(size_t nobj, size_t size,
		      QERR_Type msgnum, QERR_SeverityType Sev);
extern void *QbRealloc(void *p, size_t size,
		       QERR_Type msgnum, QERR_SeverityType Sev);
extern void  QbFree(void *p);
extern DMBEXPHP (void)             *QbNew(size_t size, const char *module ,
		                      QERR_SeverityType Sev = QERR_FATAL);
extern DMBEXPHP (void)             *QbCNew(size_t nobj, size_t size,
                                           const char *module,
                                           QERR_SeverityType Sev = QERR_FATAL);
extern DMBEXPHP (void)             QbError(QERR_Type msgnum,
                                           QERR_SeverityType Sev, ...);
extern DMBEXPHP (void)             QbDelete(void *p);
extern DMBEXPHP (int)              SetErrorMSGNumber(QERR_Type);
extern DMBEXPHP (int)              SetErrorMSGSeverity(QERR_SeverityType);
extern DMBEXPHP (int)              SetErrorMSG(char *);
extern DMBEXPHP(QERR_Type)         GetErrorMSGNumber(void);
extern DMBEXPHP(QERR_SeverityType) GetErrorMSGSeverity(void);
extern DMBEXPHP(char*)             GetErrorMSG(void);
extern DMBEXPHP(void)              QbPrintErrorMessage(FILE *stream);
#endif
