/*
 *  IBM Confidential
 *  
 *  Licensed Materials - Property of IBM
 *
 *  (C) COPYRIGHT IBM Corp. 1994-1998 All rights reserved
 *
 *  US Government Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 * Filename : qbictcl.cpp
 * Date     : 10/27/98 17:01:08
 * Version  : 1.5
 * Author   : Xiaoming Zhu
 * Purpose  : program to provide tcl script capability to qbic
 *
 */

static char sccsid[] = "@(#)qbictcl.cpp	1.5 10/27/98" ; 

static const char Copyright[] = {
  "Licensed Materials - Property of IBM\n"
  "(C) Copyright IBM Corp. 1994-1998  All rights Reserved\n"
  "US Government Users Restricted Rights - Use, duplication or\n"
  "disclosure restricted by GSA ADP Schedule Contract with IBM Corp." } ;

/* End of Common Block                                                  */
#include <tcl.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#ifndef __MWERKS__
#include <stdio.h>
#include <sys/types.h>
#else
#include <SIOUX.h>
#include <types.h>
#include <tcl.h>
#include <tclInt.h>
//#include <tclPort.h>
#include <tclMac.h>
#include <tclMacInt.h>
static int MacintoshInit();
short InstallConsole _ANSI_ARGS_((short fd));
#endif
#include <errno.h>
#include <time.h>
#ifdef _WIN32
#include <direct.h>
#include <winsock2.h>
#include <windows.h>
#include <winreg.h>
#include <process.h>
#include <MMSystem.h>
char *getRegistryPath(char *key);
#elif !defined (__MWERKS__)
typedef unsigned char   u_char;
typedef unsigned short  u_short;
typedef unsigned long   u_long;
typedef unsigned int    u_int;
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <unistd.h>
#endif
#include "QbicWrap.hpp"

static unsigned int initialized = 0;
Tcl_HashTable tableQbicWrap;

#define CONCAT(x,y) y##x
#define TCLCREATE(x) Tcl_CreateCommand(interp,#x,CONCAT(Cmd,x),0,0)

#define begintcl(NAME) int CONCAT(Cmd,NAME)(ClientData, Tcl_Interp *interp, int argc, char **argv)

int appinit(Tcl_Interp *interp);


int main(int argc, char **argv) {
/*
    if(argc > 1) {
      setQbicEnv(argv[1]);
    }
*/
#ifdef __MWERKS__
    if (MacintoshInit()  != TCL_OK) {
        Tcl_Exit(1);
    }
#endif
    Tcl_Main(argc, argv, appinit);
    return 0;
}

begintcl(qbic_clock) {
    clock_t timec;
    float timei;

    timec = clock();
    timei = (float) timec / (float) CLOCKS_PER_SEC * (float) 1000.;
    sprintf(interp->result, "%f", timei);
    return TCL_OK;
}

begintcl(qbic_start) {
    static unsigned int id = 1;
    QbicWrapClass *qbicMake;
    int again;
    Tcl_HashEntry *entryPtr;

    if(argc != 4 && argc != 1) {
      interp->result = "Usage: qbic_start [catalogName databaseName mode]";
      return TCL_ERROR;
    } else if(argc == 4) {
      qbicMake = new QbicWrapClass(argv[1], argv[2], argv[3]);
    } else {
      qbicMake = new QbicWrapClass();
    }

    if(!qbicMake) {
       interp->result = "Can't Create the Qbic Object";
       return TCL_ERROR;
    } else if (argc == 4) {
       qbicMake->QbicWrapClassConnect(argv[1], argv[2], argv[3]);
    }

    if(!initialized) {
       Tcl_InitHashTable(&tableQbicWrap, TCL_STRING_KEYS);
       initialized = 1;
    }
    
    do {
       sprintf(interp->result, "qbic%u", id);
       entryPtr = Tcl_CreateHashEntry(&tableQbicWrap, interp->result, &again);
       id++;
    } while(!again);

    
    Tcl_SetHashValue(entryPtr, qbicMake);
    return TCL_OK;
}

begintcl(qbic_add_feature) {
    QbicWrapClass *qbicMake;
    Tcl_HashEntry *entryPtr;
    int rc;

    if(argc != 3) {
      interp->result = "Usage: qbic_add_feature qbicDes featureName";
      return TCL_ERROR;
    }

    if(!initialized) {
      interp->result = "qbic is not initialized";
      return TCL_ERROR;
    }
    entryPtr = Tcl_FindHashEntry(&tableQbicWrap, argv[1]);
    if(entryPtr == NULL) {
      Tcl_AppendResult(interp,
           "No qbic object found, have you called qbic_start yet?",
           (char *) NULL);
      return TCL_ERROR;
    }
    qbicMake = (QbicWrapClass *) Tcl_GetHashValue(entryPtr);
    if(!qbicMake) {
      Tcl_AppendResult(interp,
           "qbic object is not resolved properly, have you called qbic_start?",
           (char *) NULL);
      return TCL_ERROR;
    }

    rc = qbicMake->InsertFeatures(argv[2]);
    if(rc) {
      Tcl_AppendResult(interp,
           "Cannot insert feature ", argv[2], "Due to ", 
           GetErrorMSG(), (char *) NULL);
      return TCL_ERROR;
    }
    Tcl_AppendResult(interp, "feature ", argv[2], " added", (char *) NULL);

    return TCL_OK;
}

begintcl(qbic_delete_feature) {
    QbicWrapClass *qbicMake;
    Tcl_HashEntry *entryPtr;
    int rc;

    if(argc != 3) {
      interp->result = "Usage: qbic_delete_feature qbicDes featureName";
      return TCL_ERROR;
    }

    if(!initialized) {
      interp->result = "qbic is not initialized";
      return TCL_ERROR;
    }
    entryPtr = Tcl_FindHashEntry(&tableQbicWrap, argv[1]);
    if(entryPtr == NULL) {
      Tcl_AppendResult(interp,
           "No qbic object found, have you called qbic_start yet?",
           (char *) NULL);
      return TCL_ERROR;
    }
    qbicMake = (QbicWrapClass *) Tcl_GetHashValue(entryPtr);
    if(!qbicMake) {
      Tcl_AppendResult(interp,
           "qbic object is not resolved properly, have you called qbic_start?",
           (char *) NULL);
      return TCL_ERROR;
    }

    rc = qbicMake->DeleteFeatures(argv[2]);
    if(rc) {
      Tcl_AppendResult(interp,
           "Cannot delete feature ", argv[2], " ", (char *) NULL);
      return TCL_ERROR;
    }
    Tcl_AppendResult(interp, "feature ", argv[2], " deleted", (char *) NULL);

    return TCL_OK;
}

begintcl(qbic_image_fdata) {
    QbicWrapClass *qbicMake;
    Tcl_HashEntry *entryPtr;
    QbStringClass str(800);
    int rc;

    if(argc < 3) {
      interp->result = "Usage: qbic_image_fdata qbicDes imgName [maskName]";
      return TCL_ERROR;
    }

    if(!initialized) {
      interp->result = "qbic is not initialized";
      return TCL_ERROR;
    }
    entryPtr = Tcl_FindHashEntry(&tableQbicWrap, argv[1]);
    if(entryPtr == NULL) {
      Tcl_AppendResult(interp,
           "No qbic object found, have you called qbic_start yet?",
           (char *) NULL);
      return TCL_ERROR;
    }
    qbicMake = (QbicWrapClass *) Tcl_GetHashValue(entryPtr);
    if(!qbicMake) {
      Tcl_AppendResult(interp,
           "qbic object is not resolved properly, have you called qbic_start?",
           (char *) NULL);
      return TCL_ERROR;
    }

    if(argc == 3) rc = qbicMake->GetFeatureDataImage(argv[2], NULL, &str);
    else          rc = qbicMake->GetFeatureDataImage(argv[2], argv[3], &str);
    if(rc) {
      Tcl_AppendResult(interp,
           "Cannot get feature data", argv[2], " ", (char *) NULL);
      return TCL_ERROR;
    }
    Tcl_AppendResult(interp, "key=", argv[2], " \n", (char *) str,
                     (char *) NULL);

    return TCL_OK;
}

begintcl(qbic_key_fdata) {
    QbicWrapClass *qbicMake;
    Tcl_HashEntry *entryPtr;
    QbStringClass str(800);
    int rc;

    if(argc != 3) {
      interp->result = "Usage: qbic_key_fdata qbicDes keyName";
      return TCL_ERROR;
    }

    if(!initialized) {
      interp->result = "qbic is not initialized";
      return TCL_ERROR;
    }
    entryPtr = Tcl_FindHashEntry(&tableQbicWrap, argv[1]);
    if(entryPtr == NULL) {
      Tcl_AppendResult(interp,
           "No qbic object found, have you called qbic_start yet?",
           (char *) NULL);
      return TCL_ERROR;
    }
    qbicMake = (QbicWrapClass *) Tcl_GetHashValue(entryPtr);
    if(!qbicMake) {
      Tcl_AppendResult(interp,
           "qbic object is not resolved properly, have you called qbic_start?",
           (char *) NULL);
      return TCL_ERROR;
    }

    rc = qbicMake->GetFeatureDataKey(argv[2], &str);
    if(rc) {
      Tcl_AppendResult(interp,
           "Cannot get feature data", argv[2], " ", (char *) NULL);
      return TCL_ERROR;
    }
    Tcl_AppendResult(interp, "key=", argv[2], " \n", (char *) str,
                     (char *) NULL);

    return TCL_OK;
}

begintcl(qbic_picker_fdata) {
    QbicWrapClass *qbicMake;
    Tcl_HashEntry *entryPtr;
    QbStringClass str(800);
    int rc;

    if(argc < 3) {
      interp->result = "Usage: qbic_picker_fdata qbicDes imgName [maskName]";
      return TCL_ERROR;
    }

    if(!initialized) {
      interp->result = "qbic is not initialized";
      return TCL_ERROR;
    }
    entryPtr = Tcl_FindHashEntry(&tableQbicWrap, argv[1]);
    if(entryPtr == NULL) {
      Tcl_AppendResult(interp,
           "No qbic object found, have you called qbic_start yet?",
           (char *) NULL);
      return TCL_ERROR;
    }
    qbicMake = (QbicWrapClass *) Tcl_GetHashValue(entryPtr);
    if(!qbicMake) {
      Tcl_AppendResult(interp,
           "qbic object is not resolved properly, have you called qbic_start?",
           (char *) NULL);
      return TCL_ERROR;
    }

    if(argc == 3) rc = qbicMake->GetFeatureDataPicker(argv[2], NULL, &str);
    else          rc = qbicMake->GetFeatureDataPicker(argv[2], argv[3], &str);
    if(rc) {
      Tcl_AppendResult(interp,
           "Cannot get feature data", argv[2], " ", (char *) NULL);
      return TCL_ERROR;
    }
    Tcl_AppendResult(interp, "key=", argv[2], " \n", (char *) str,
                     (char *) NULL);

    return TCL_OK;
}

begintcl(qbic_string_fdata) {
    QbicWrapClass *qbicMake;
    Tcl_HashEntry *entryPtr;
    QbStringClass str(800);
    int rc;

    if(argc < 3) {
      interp->result = "Usage: qbic_string_fdata qbicDes stringDes";
      return TCL_ERROR;
    }

    if(!initialized) {
      interp->result = "qbic is not initialized";
      return TCL_ERROR;
    }
    entryPtr = Tcl_FindHashEntry(&tableQbicWrap, argv[1]);
    if(entryPtr == NULL) {
      Tcl_AppendResult(interp,
           "No qbic object found, have you called qbic_start yet?",
           (char *) NULL);
      return TCL_ERROR;
    }
    qbicMake = (QbicWrapClass *) Tcl_GetHashValue(entryPtr);
    if(!qbicMake) {
      Tcl_AppendResult(interp,
           "qbic object is not resolved properly, have you called qbic_start?",
           (char *) NULL);
      return TCL_ERROR;
    }

    rc = qbicMake->GetFeatureDataString(argv[2], &str);
    if(rc) {
      Tcl_AppendResult(interp,
           "Cannot get feature data", argv[2], " ", (char *) NULL);
      return TCL_ERROR;
    }
    Tcl_AppendResult(interp, "key=", argv[2], " \n", (char *) str,
                     (char *) NULL);

    return TCL_OK;
}

begintcl(qbic_add_image) {
    QbicWrapClass *qbicMake;
    Tcl_HashEntry *entryPtr;
    char *value;
    int rc;

    if(argc != 3) {
      interp->result = "Usage: qbic_add_image qbicDes imageName";
      return TCL_ERROR;
    }

    if(!initialized) {
      interp->result = "qbic is not initialized";
      return TCL_ERROR;
    }
    entryPtr = Tcl_FindHashEntry(&tableQbicWrap, argv[1]);
    if(entryPtr == NULL) {
      Tcl_AppendResult(interp,
           "No qbic object found, have you called qbic_start yet?",
           (char *) NULL);
      return TCL_ERROR;
    }
    qbicMake = (QbicWrapClass *) Tcl_GetHashValue(entryPtr);
    if(!qbicMake) {
      Tcl_AppendResult(interp,
           "qbic object is not resolved properly, have you called qbic_start?",
           (char *) NULL);
      return TCL_ERROR;
    }

    value = Tcl_GetVar2(interp, "env", "QbicTextPath", 0);
    Qbsetenv("QbicTextPath", value);
    rc = qbicMake->QbicWrapInsertImage(argv[2]);
    if(rc) {
      Tcl_AppendResult(interp,
           "Cannot add image ", argv[2], "Due to ", GetErrorMSG(), 
           (char *) NULL);
      return TCL_ERROR;
    }
    Tcl_AppendResult(interp, "image ", argv[2], " added successfully", 
                     (char *) NULL);

    return TCL_OK;
}

begintcl(qbic_delete_image) {
    QbicWrapClass *qbicMake;
    Tcl_HashEntry *entryPtr;
    int rc;

    if(argc != 3) {
      interp->result = "Usage: qbic_delete_image qbicDes imageName";
      return TCL_ERROR;
    }

    if(!initialized) {
      interp->result = "qbic is not initialized";
      return TCL_ERROR;
    }
    entryPtr = Tcl_FindHashEntry(&tableQbicWrap, argv[1]);
    if(entryPtr == NULL) {
      Tcl_AppendResult(interp,
           "No qbic object found, have you called qbic_start yet?",
           (char *) NULL);
      return TCL_ERROR;
    }
    qbicMake = (QbicWrapClass *) Tcl_GetHashValue(entryPtr);
    if(!qbicMake) {
      Tcl_AppendResult(interp,
           "qbic object is not resolved properly, have you called qbic_start?",
           (char *) NULL);
      return TCL_ERROR;
    }

    rc = qbicMake->QbicWrapDeleteImage(argv[2]);
    if(rc) {
      Tcl_AppendResult(interp,
           "Cannot delete image ", argv[2], " ", (char *) NULL);
      return TCL_ERROR;
    }
    Tcl_AppendResult(interp, "image ", argv[2], " deleted", (char *) NULL);

    return TCL_OK;
}

begintcl(qbic_query_key) {
    QbicWrapClass *qbicMake;
    QbStringClass str(800);
    Tcl_HashEntry *entryPtr;
    int rc;

    if(argc < 3) {
      interp->result = 
          "Usage: qbic_query_key qbicDes imageKey";
      return TCL_ERROR;
    }

    if(!initialized) {
      interp->result = "qbic is not initialized";
      return TCL_ERROR;
    }
    entryPtr = Tcl_FindHashEntry(&tableQbicWrap, argv[1]);
    if(entryPtr == NULL) {
      Tcl_AppendResult(interp,
           "No qbic object found, have you called qbic_start yet?",
           (char *) NULL);
      return TCL_ERROR;
    }
    qbicMake = (QbicWrapClass *) Tcl_GetHashValue(entryPtr);
    if(!qbicMake) {
      Tcl_AppendResult(interp,
           "qbic object is not resolved properly, have you called qbic_start?",
           (char *) NULL);
      return TCL_ERROR;
    }

    rc = qbicMake->QbicWrapQueryKey(argv[2]);
    if(rc) {
      Tcl_AppendResult(interp,
           "Cannot read image ", argv[2], " ", (char *) NULL);
      return TCL_ERROR;
    }
    rc = qbicMake->QbicWrapQueryDB(&str);
    if(rc) {
       printf("Error=%d  Severity=%d\n%s \n",
            GetErrorMSGNumber(), GetErrorMSGSeverity(), GetErrorMSG() ) ;
       return TCL_ERROR;
    }
    if(str.IsOK()) Tcl_AppendResult(interp, (char *) str, (char *) NULL);
    else return TCL_ERROR;
    return TCL_OK;
}

begintcl(qbic_query_image) {
    QbicWrapClass *qbicMake;
    QbStringClass str(800);
    Tcl_HashEntry *entryPtr;
    int rc;

    if(argc < 3) {
      interp->result = 
          "Usage: qbic_query_image qbicDes imageName [maskName]";
      return TCL_ERROR;
    }

    if(!initialized) {
      interp->result = "qbic is not initialized";
      return TCL_ERROR;
    }
    entryPtr = Tcl_FindHashEntry(&tableQbicWrap, argv[1]);
    if(entryPtr == NULL) {
      Tcl_AppendResult(interp,
           "No qbic object found, have you called qbic_start yet?",
           (char *) NULL);
      return TCL_ERROR;
    }
    qbicMake = (QbicWrapClass *) Tcl_GetHashValue(entryPtr);
    if(!qbicMake) {
      Tcl_AppendResult(interp,
           "qbic object is not resolved properly, have you called qbic_start?",
           (char *) NULL);
      return TCL_ERROR;
    }

    if(argc > 3) rc = qbicMake->QbicWrapQueryImage(argv[2], argv[3]);
    else         rc = qbicMake->QbicWrapQueryImage(argv[2], NULL);
    if(rc) {
      Tcl_AppendResult(interp,
           "Cannot read image ", argv[2], " ", (char *) NULL);
      return TCL_ERROR;
    }
    rc = qbicMake->QbicWrapQueryDB(&str);
    if(rc) {
       printf("Error=%d  Severity=%d\n%s \n",
            GetErrorMSGNumber(), GetErrorMSGSeverity(), GetErrorMSG() ) ;
       return TCL_ERROR;
    }
    if(str.IsOK()) Tcl_AppendResult(interp, (char *) str, (char *) NULL);
    else return TCL_ERROR;
    return TCL_OK;
}

begintcl(qbic_make_thumb) {
    QbicWrapClass *qbicMake;
    Tcl_HashEntry *entryPtr;
    int rc;

    if(argc != 4) {
      interp->result = "Usage: qbic_make_thumb qbicDes image_in image_out";
      return TCL_ERROR;
    }

    if(!initialized) {
      interp->result = "qbic is not initialized";
      return TCL_ERROR;
    }
    entryPtr = Tcl_FindHashEntry(&tableQbicWrap, argv[1]);
    if(entryPtr == NULL) {
      Tcl_AppendResult(interp,
           "No qbic object found, have you called qbic_start yet?",
           (char *) NULL);
      return TCL_ERROR;
    }
    qbicMake = (QbicWrapClass *) Tcl_GetHashValue(entryPtr);
    if(!qbicMake) {
      Tcl_AppendResult(interp,
           "qbic object is not resolved properly, have you called qbic_start?",
           (char *) NULL);
      return TCL_ERROR;
    }

    rc = qbicMake->QbicWrapThumb(argv[2], argv[3]);
    if(rc) {
       Tcl_AppendResult(interp, "Create ", argv[3], " Failed due to ",
                        GetErrorMSG(), (char *) NULL);
       return TCL_ERROR;
    } else {
       Tcl_AppendResult(interp, "Create ", argv[3], " Successful", NULL);
       return TCL_OK;
    }
    return TCL_OK;
}

begintcl(qbic_set_thumb_pad)
{
    QbicWrapClass *qbicMake;
    Tcl_HashEntry *entryPtr;

    if(argc != 2 && argc != 3) {
      interp->result = "Usage: qbic_set_thumb_pad qbicDes ?True?";
      return TCL_ERROR;
    }

    if(!initialized) {
      interp->result = "qbic is not initialized";
      return TCL_ERROR;
    }
    entryPtr = Tcl_FindHashEntry(&tableQbicWrap, argv[1]);
    if(entryPtr == NULL) {
      Tcl_AppendResult(interp, "No qbic descriptor", (char *) NULL);
      return TCL_ERROR;
    }
    qbicMake = (QbicWrapClass *) Tcl_GetHashValue(entryPtr);
    if(argc == 2) qbicMake->QbicWrapSetPad(False);
    else {
       if(!strcmp(argv[2], "True") ) qbicMake->QbicWrapSetPad(True);
       else                          qbicMake->QbicWrapSetPad(False);
    }
    return TCL_OK;
}

begintcl(qbic_set_keys_return) {
    QbicWrapClass *qbicMake;
    Tcl_HashEntry *entryPtr;
    int rc;

    if(argc != 3) {
      interp->result = "Usage: qbic_set_keys_return qbicDes num";
      return TCL_ERROR;
    }

    if(!initialized) {
      interp->result = "qbic is not initialized";
      return TCL_ERROR;
    }
    entryPtr = Tcl_FindHashEntry(&tableQbicWrap, argv[1]);
    if(entryPtr == NULL) {
      Tcl_AppendResult(interp,
           "No qbic object found, have you called qbic_start yet?",
           (char *) NULL);
      return TCL_ERROR;
    }
    qbicMake = (QbicWrapClass *) Tcl_GetHashValue(entryPtr);
    if(!qbicMake) {
      Tcl_AppendResult(interp,
           "qbic object is not resolved properly, have you called qbic_start?",
           (char *) NULL);
      return TCL_ERROR;
    }

    rc = atoi(argv[2]);
    rc = qbicMake->QbicWrapSetReturnedKeys(rc);
    Tcl_AppendResult(interp, "set returned number of query keys=", argv[2],
                     (char *) NULL);

    return TCL_OK;
}

begintcl(qbic_make_connect) {
    QbicWrapClass *qbicMake;
    Tcl_HashEntry *entryPtr;
    int rc;

    if(argc < 4) {
      interp->result = "Usage: qbic_make_connect qbicDes catName databaseName [mode]";
      return TCL_ERROR;
    }

    if(!initialized) {
      interp->result = "qbic is not initialized";
      return TCL_ERROR;
    }
    entryPtr = Tcl_FindHashEntry(&tableQbicWrap, argv[1]);
    if(entryPtr == NULL) {
      Tcl_AppendResult(interp,
           "No qbic object found, have you called qbic_start yet?",
           (char *) NULL);
      return TCL_ERROR;
    }
    qbicMake = (QbicWrapClass *) Tcl_GetHashValue(entryPtr);
    if(!qbicMake) {
      Tcl_AppendResult(interp,
           "qbic object is not resolved properly, have you called qbic_start?",
           (char *) NULL);
      return TCL_ERROR;
    }

    if(argc == 4) rc = qbicMake->QbicWrapClassConnect(argv[2], argv[3], "r");
    else rc = qbicMake->QbicWrapClassConnect(argv[2], argv[3], argv[4]);

    if(rc) {
       Tcl_AppendResult(interp, "Can't establish the connection with catalog ",
           argv[2], "in database ", argv[3], (char *) NULL);
       return TCL_ERROR;
    }
    return TCL_OK;
}

begintcl(qbic_get_cat_records) {
    QbicWrapClass *qbicMake;
    Tcl_HashEntry *entryPtr;
    char          buffer[20];
    int rc;

    if(argc < 3) {
      interp->result = "Usage: qbic_cat_records qbicDes FeatureName";
      return TCL_ERROR;
    }

    if(!initialized) {
      interp->result = "qbic is not initialized";
      return TCL_ERROR;
    }
    entryPtr = Tcl_FindHashEntry(&tableQbicWrap, argv[1]);
    if(entryPtr == NULL) {
      Tcl_AppendResult(interp,
           "No qbic object found, have you called qbic_start yet?",
           (char *) NULL);
      return TCL_ERROR;
    }
    qbicMake = (QbicWrapClass *) Tcl_GetHashValue(entryPtr);
    if(!qbicMake) {
      Tcl_AppendResult(interp,
           "qbic object is not resolved properly, have you called qbic_start?",
           (char *) NULL);
      return TCL_ERROR;
    }

    rc = qbicMake->ListCatRecords(argv[2]);
    if(rc < 0) {
      Tcl_AppendResult(interp,
           "Cannot Get Number of Record for the feature named", argv[2], 
            " due to ", GetErrorMSG(), (char *) NULL);
      return TCL_ERROR;
    }
    sprintf(buffer, "%d", rc);
    Tcl_AppendResult(interp, buffer, (char *) NULL);
    QbDelete(buffer);

    return TCL_OK;
}

begintcl(qbic_get_param) {
    QbicWrapClass *qbicMake;
    Tcl_HashEntry *entryPtr;
    QbFeatureClass *feature = NULL;
    QbDatumClass   datum;
    QbParameterClass *parm = NULL;
    unsigned long  size;
    char           *buffer;
    int rc;

    if(argc < 2) {
      interp->result = "Usage: qbic_get_param FeatureClassName [qbicDes]";
      return TCL_ERROR;
    }

    if(argc == 3) {
      if(!initialized) {
        interp->result = "qbic is not initialized";
        return TCL_ERROR;
      }
      entryPtr = Tcl_FindHashEntry(&tableQbicWrap, argv[2]);
      if(entryPtr == NULL) {
        Tcl_AppendResult(interp,
           "No qbic object found, have you called qbic_start yet?",
           (char *) NULL);
        return TCL_ERROR;
      }
      qbicMake = (QbicWrapClass *) Tcl_GetHashValue(entryPtr);
      if(!qbicMake) {
        Tcl_AppendResult(interp,
           "qbic object is not resolved properly, have you called qbic_start?",
           (char *) NULL);
        return TCL_ERROR;
      }
      parm = qbicMake->GetParamForFeature(argv[1]);
    }
    if(!parm) {
      feature = QbCreateFeatureClass(argv[1]);
      if(!feature) {
        Tcl_AppendResult(interp,
           "Cannot Create Feature Class", argv[1], (char *) NULL);
        return TCL_ERROR;
      }
      parm = (QbParameterClass *) feature->GetDefaultParameters();
    }
    rc = parm->ToCommandLineString(datum);
    if(rc) {
      Tcl_AppendResult(interp,
           "Cannot Get Parameter for ", argv[1], " due to ", GetErrorMSG(),
           (char *) NULL);
      return TCL_ERROR;
    }
    datum.Get(size, buffer);
    Tcl_AppendResult(interp, argv[1], ":", buffer, (char *) NULL);
    if(feature) delete feature;

    return TCL_OK;
}

begintcl(qbic_list_cat_features) {
    QbicWrapClass *qbicMake;
    Tcl_HashEntry *entryPtr;
    char          *buffer;

    if(argc < 2) {
      interp->result = "Usage: qbic_list_cat_features qbicDes";
      return TCL_ERROR;
    }

    if(!initialized) {
      interp->result = "qbic is not initialized";
      return TCL_ERROR;
    }
    entryPtr = Tcl_FindHashEntry(&tableQbicWrap, argv[1]);
    if(entryPtr == NULL) {
      Tcl_AppendResult(interp,
           "No qbic object found, have you called qbic_start yet?",
           (char *) NULL);
      return TCL_ERROR;
    }
    qbicMake = (QbicWrapClass *) Tcl_GetHashValue(entryPtr);
    if(!qbicMake) {
      Tcl_AppendResult(interp,
           "qbic object is not resolved properly, have you called qbic_start?",
           (char *) NULL);
      return TCL_ERROR;
    }

    buffer = qbicMake->ListCatFeatures();
    if(!buffer) {
      Tcl_AppendResult(interp,
           "Cannot Get Feature List due to ", GetErrorMSG(), (char *) NULL);
      return TCL_ERROR;
    }
    Tcl_AppendResult(interp, buffer, (char *) NULL);
    QbDelete(buffer);

    return TCL_OK;
}

begintcl(qbic_list_all_features) {
    QbicWrapClass *qbicMake;
    Tcl_HashEntry *entryPtr;
    char          *buffer;

    qbicMake = new QbicWrapClass; // we have to do this to please MAC
    if(!qbicMake) return TCL_ERROR;
    buffer = qbicMake->ListAllFeatures();
    if(!buffer) {
      Tcl_AppendResult(interp,
           "Cannot Get Feature List", (char *) NULL);
      return TCL_ERROR;
    }
    Tcl_AppendResult(interp, buffer, (char *) NULL);
    QbDelete(buffer);
    delete qbicMake;

    return TCL_OK;
}

begintcl(QbQBE)
{
    QbicWrapClass *qbicMake;
    QbStringClass str(800);
    int rc;

    qbicMake = new QbicWrapClass;
    if(!qbicMake) return TCL_ERROR;
    qbicMake->SetScreenPrint(0);
    rc = qbicMake->QbicWrapQBE(argc, argv, &str);
    if(rc || !str.IsOK()) {
       printf("Error=%d  Severity=%d\n%s \n",
            GetErrorMSGNumber(), GetErrorMSGSeverity(), GetErrorMSG() ) ;
       return TCL_ERROR;
    }
    Tcl_AppendResult(interp, (char *) str, (char *) NULL);
    delete qbicMake;
    return TCL_OK;
}

begintcl(QbMkDbs)
{
    QbicWrapClass *qbicMake;
    QbStringClass str(200);
    int rc;

    qbicMake = new QbicWrapClass;
    if(!qbicMake) return TCL_ERROR;
    qbicMake->SetScreenPrint(1);
    rc = qbicMake->QbicWrapDBS(argc, argv, &str);
    if(rc) {
      printf("Error=%d  Severity=%d\n%s \n",
            GetErrorMSGNumber(), GetErrorMSGSeverity(), GetErrorMSG() ) ;
      Tcl_AppendResult(interp,
           "QbMkDbs failed due to ", GetErrorMSG(), (char *) NULL);
       return TCL_ERROR;
    }
    Tcl_AppendResult(interp, (char *) str, (char *) NULL);
    delete qbicMake;
    return TCL_OK;
}

begintcl(QbMkThmb)
{
    QbicWrapClass *qbicMake;
    QbStringClass str(200);
    int rc;

    qbicMake = new QbicWrapClass;
    qbicMake->SetScreenPrint(0);
    if(!qbicMake) return TCL_ERROR;
    rc = qbicMake->QbicWrapThm(argc, argv, &str);
    if(rc) {
      printf("Error=%d  Severity=%d\n%s \n",
            GetErrorMSGNumber(), GetErrorMSGSeverity(), GetErrorMSG() ) ;
      Tcl_AppendResult(interp,
           "QbMkThmb failed due to ", GetErrorMSG(), (char *) NULL);
       return TCL_ERROR;
    }
    Tcl_AppendResult(interp, (char *) str, (char *) NULL);
    delete qbicMake;
    return TCL_OK;
}

begintcl(QbDumpDb)
{
    QbicWrapClass *qbicMake;
    int rc;

    qbicMake = new QbicWrapClass;
    if(!qbicMake) return TCL_ERROR;
    rc = qbicMake->QbicWrapDumpDb(argc, argv, stdout);
    if(rc) {
      printf("Error=%d  Severity=%d\n%s \n",
            GetErrorMSGNumber(), GetErrorMSGSeverity(), GetErrorMSG() ) ;
      Tcl_AppendResult(interp,
           "QbDumpdb failed due to ", GetErrorMSG(), (char *) NULL);
       return TCL_ERROR;
    }
    //Tcl_AppendResult(interp, (char *) str, (char *) NULL);
    delete qbicMake;
    return TCL_OK;
}


begintcl(qbic_set_thumb_size)
{
    QbicWrapClass *qbicMake;
    Tcl_HashEntry *entryPtr;

    if(argc != 2 && argc != 4) {
      interp->result = "Usage: qbic_set_thumb_size qbicDes [width, height]";
      return TCL_ERROR;
    }

    if(!initialized) {
      interp->result = "qbic is not initialized";
      return TCL_ERROR;
    }
    entryPtr = Tcl_FindHashEntry(&tableQbicWrap, argv[1]);
    if(entryPtr == NULL) {
      Tcl_AppendResult(interp, "No qbic descriptor", (char *) NULL);
      return TCL_ERROR;
    }
    qbicMake = (QbicWrapClass *) Tcl_GetHashValue(entryPtr);
    if(argc == 2) qbicMake->QbicWrapThumbXY(100, 100);
    else         qbicMake->QbicWrapThumbXY(atoi(argv[2]), atoi(argv[3]));
    return TCL_OK;
}

begintcl(qbic_set_thumb_24color)
{
    QbicWrapClass *qbicMake;
    Tcl_HashEntry *entryPtr;

    if(argc != 2 && argc != 3) {
      interp->result = "Usage: qbic_set_thumb_24color qbicDes [True]";
      return TCL_ERROR;
    }

    if(!initialized) {
      interp->result = "qbic is not initialized";
      return TCL_ERROR;
    }
    entryPtr = Tcl_FindHashEntry(&tableQbicWrap, argv[1]);
    if(entryPtr == NULL) {
      Tcl_AppendResult(interp, "No qbic descriptor", (char *) NULL);
      return TCL_ERROR;
    }
    qbicMake = (QbicWrapClass *) Tcl_GetHashValue(entryPtr);
    if(argc == 2) qbicMake->QbicWrapSetThumb24Color(False);
    else {
       if(!strcmp(argv[2], "True") ) qbicMake->QbicWrapSetThumb24Color(True);
       else                          qbicMake->QbicWrapSetThumb24Color(False);
    }
    return TCL_OK;
}

begintcl(qbic_set_keyword)
{
    char None='\0';
    QbicWrapClass *qbicMake;
    Tcl_HashEntry *entryPtr;

    if(argc < 3) {
      interp->result = "Usage: qbic_set_keyword qbicDes [keyword]";
      return TCL_ERROR;
    }

    if(!initialized) {
      interp->result = "qbic is not initialized";
      return TCL_ERROR;
    }
    entryPtr = Tcl_FindHashEntry(&tableQbicWrap, argv[1]);
    if(entryPtr == NULL) {
      Tcl_AppendResult(interp, "No file open for close", (char *) NULL);
      return TCL_ERROR;
    }
    qbicMake = (QbicWrapClass *) Tcl_GetHashValue(entryPtr);
    if(argc > 2) qbicMake->QbicWrapSetKeyWord(argv[2]);
    else         qbicMake->QbicWrapSetKeyWord(&None);
    return TCL_OK;
}

begintcl(qbic_close)
{
    QbicWrapClass *qbicMake;
    Tcl_HashEntry *entryPtr;

    if(argc != 2) {
      interp->result = "Usage: qbic_close qbicDes";
      return TCL_ERROR;
    }

    if(!initialized) {
      interp->result = "qbic is not initialized";
      return TCL_ERROR;
    }
    entryPtr = Tcl_FindHashEntry(&tableQbicWrap, argv[1]);
    if(entryPtr == NULL) {
      Tcl_AppendResult(interp, "No qbic descriptor", (char *) NULL);
      return TCL_ERROR;
    }
    qbicMake = (QbicWrapClass *) Tcl_GetHashValue(entryPtr);
    qbicMake->QbicWrapCleanUp();
    return TCL_OK;
}

begintcl(qbic_end)
{
    QbicWrapClass *qbicMake;
    Tcl_HashEntry *entryPtr;

    if(argc != 2) {
      interp->result = "Usage: qbic_end qbicDescriptor";
      return TCL_ERROR;
    }

    if(!initialized) {
      interp->result = "qbic is not initialized";
      return TCL_ERROR;
    }
    entryPtr = Tcl_FindHashEntry(&tableQbicWrap, argv[1]);
    if(entryPtr == NULL) {
      Tcl_AppendResult(interp, "No qbic descriptor exists for close", 
                       (char *) NULL);
      return TCL_ERROR;
    }
    qbicMake = (QbicWrapClass *) Tcl_GetHashValue(entryPtr);
    Tcl_DeleteHashEntry(entryPtr);
    if(qbicMake) delete qbicMake;
    
    return TCL_OK;
}

#ifdef _WIN32
char *getRegistryPath(char *key)
{
HKEY hKey, hSubKey;
LONG rc = !ERROR_SUCCESS;
int  idx;
char *cpt = key, *lpMsgBuf;
unsigned char *value;
DWORD dwType, dwSize;

    idx = strcspn(cpt, "\\");
    cpt[idx] = '\0';

    if(!strncmp(cpt, "HKEY_LOCAL_MACHINE", idx)) {
       hKey = HKEY_LOCAL_MACHINE;
    }
    else if (!strncmp(cpt, "HKEY_CLASSESE_ROOT", idx)) {
       hKey = HKEY_CLASSES_ROOT;
    }
    else if (!strncmp(cpt, "HKEY_USERS", idx)) {
       hKey = HKEY_USERS;
    }
    else if (!strncmp(cpt, "HKEY_CURRENT_USER", idx)) {
       hKey = HKEY_CURRENT_USER;
    }
    else {
       printf("registry search for %s is not supported\n", cpt);
    }

    cpt[idx] = '\\';
    cpt += idx + 1;
    idx = strcspn(cpt, "\\");
    while(cpt[idx] == '\\') {
       cpt[idx] = '\0';
       rc = RegOpenKeyEx(hKey, cpt, 0, KEY_READ, &hSubKey);
       if(rc == ERROR_SUCCESS) {
          RegCloseKey(hKey);
          hKey = hSubKey;
       } else {
          RegCloseKey(hKey);
          hKey = NULL;
          break;
       }
       cpt[idx] = '\\';
       cpt += idx + 1;
       idx = strcspn(cpt, "\\");
    }
    if(hKey == NULL) {
       printf("invalid key %s\n", key);
       return NULL;
    } else {
       rc = RegOpenKeyEx(hKey, cpt, 0, KEY_READ, &hSubKey);
       RegCloseKey(hKey);
       hKey = hSubKey;
    }
    rc = RegQueryValueEx(hKey, "", NULL, &dwType, NULL, &dwSize);
    if(rc==ERROR_SUCCESS) {
      value = (unsigned char *) malloc(dwSize);
      if(!value) {
         RegCloseKey(hKey);
         printf("can't malloc buffer of size %d\n", dwSize);
         return NULL;
      }
      rc = RegQueryValueEx(hKey, "", NULL, &dwType, value, &dwSize);
      if(rc==ERROR_SUCCESS) {
         RegCloseKey(hKey);
         return (char *) value;
      }
      else {
         FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER |
                    FORMAT_MESSAGE_FROM_SYSTEM,
                    NULL,    GetLastError(),
                    MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
                   (LPTSTR) &lpMsgBuf,    0,    NULL );
         printf("error in reading key: %s\n", lpMsgBuf);
         RegCloseKey(hKey);
         return NULL;
      }
    } else {
      FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER |
                    FORMAT_MESSAGE_FROM_SYSTEM,
                    NULL,    GetLastError(),
                    MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
                   (LPTSTR) &lpMsgBuf,    0,    NULL );
      printf("error in reading key: %s\n", lpMsgBuf);
      RegCloseKey(hKey);
      return NULL;
    }
}
begintcl(SoundPlay) {
    char *option = NULL;
    BOOL bool;
    if(argc < 1) {
       interp->result = "Usage: SoundPlay soundFile ?wait?";
       return TCL_ERROR;
    }
    if(argc > 2 && !strcmp(argv[2], "wait")) {
       bool = PlaySound(argv[1], NULL,
                     SND_FILENAME | SND_SYNC | SND_NOSTOP | SND_NODEFAULT);
    } else {
       bool = PlaySound(argv[1], NULL,
                     SND_FILENAME | SND_ASYNC | SND_NOSTOP | SND_NODEFAULT);
    }
    return TCL_OK;
}
begintcl(SoundStop) {
    char *option = NULL;
    BOOL bool;
    bool = PlaySound(NULL, NULL,
                     SND_FILENAME | SND_ASYNC | SND_NOSTOP | SND_NODEFAULT);
    return TCL_OK;
}
#endif

#ifndef __MWERKS__
begintcl(get_host_ip) {
   char *name, hostnameP[180];
   struct hostent *hp;
#ifdef _WIN32
   WSADATA wsaData;
   int err = WSAStartup(MAKEWORD(2,0), &wsaData);
#endif

   if(argc > 1) name = argv[1];
   else {
     strcpy(hostnameP, "info hostname");
     name = hostnameP;
     if(Tcl_Eval(interp, name) == TCL_OK) {
       strcpy(name, interp->result);
     } else {
       interp->result = "Can't find hostname";
       return TCL_ERROR;
     }
   }

   hp = gethostbyname(name);

   if(hp) {
      sprintf(hostnameP, "%d.%d.%d.%d",
                         hp->h_addr_list[0][0] & 0xFF,
                         hp->h_addr_list[0][1] & 0xFF,
                         hp->h_addr_list[0][2] & 0xFF, 
                         hp->h_addr_list[0][3] & 0xFF);
      interp->result = hostnameP;
#ifdef _WIN32
      if(!err) WSACleanup();
#endif
      return TCL_OK;
   }

   else {
      interp->result = "Can't get Host Info";
#ifdef _WIN32
      if(!err) WSACleanup();
#endif
      return TCL_ERROR;
   }
}

begintcl(start_browser)
{
char *browser = "netscape", *shortName = "netscape";
int rc = 0;
#ifdef _WIN32
char *k1="HKEY_LOCAL_MACHINE\\SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\App Paths\\Netscape.exe";
char *k2="HKEY_LOCAL_MACHINE\\SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\App Paths\\IExplore.exe";

    browser = getRegistryPath(k1);
    if(!browser) browser = getRegistryPath(k2);
    if(!browser) {
      interp->result = "Neither Netscape nor IE4 is detected!";
      return TCL_OK;
    } else {
      shortName = 1 + strrchr(browser, '\\');
    }
#else
pid_t pid;
#endif

    if(argc > 2) {
      interp->result = "Usage: start_browser ?URL?";
      return TCL_ERROR;
    }

#ifdef _WIN32
    if(argc == 1) {
      spawnl(_P_NOWAIT, browser, shortName, NULL);
    } else {
      spawnle(_P_NOWAIT, browser, shortName, argv[1], NULL, NULL);
    }
    free(browser);
#else
    if ( (pid = fork()) < 0) {
       Tcl_AppendResult(interp, "Can't fork new process", (char *) NULL);
       return TCL_ERROR;
    } else if (pid == 0) {
      if(argc == 1) {
        rc = execlp(browser, shortName, NULL);
      } else {
        rc = execlp(browser, shortName, argv[1], NULL);
      }
    }
    if(rc) {
       Tcl_AppendResult(interp, (char *) strerror(errno), (char *) NULL);
       return TCL_ERROR;
    }
#endif
    return TCL_OK;
}
#endif

#ifdef __MWERKS__
/*
 *----------------------------------------------------------------------
 *
 * MacintoshInit --
 *
 *      This procedure calls initalization routines to set up a simple
 *      console on a Macintosh.  This is necessary as the Mac doesn't
 *      have a stdout & stderr by default.
 *
 * Results:
 *      Returns TCL_OK if everything went fine.  If it didn't the
 *      application should probably fail.
 *
 * Side effects:
 *      Inits the appropiate console package.
 *
 *----------------------------------------------------------------------
 */

static int
MacintoshInit()
{
#if GENERATING68K && !GENERATINGCFM
    SetApplLimit(GetApplLimit() - (TCL_MAC_68K_STACK_GROWTH));
#endif
    MaxApplZone();

#if defined(THINK_C)

    /* Set options for Think C console package */
    /* The console package calls the Mac init calls */
    console_options.pause_atexit = 0;
    console_options.title = "\pTcl Interpreter";

#elif defined(__MWERKS__)

    /* Set options for CodeWarrior SIOUX package */
    SIOUXSettings.autocloseonquit = true;
    SIOUXSettings.showstatusline = true;
    SIOUXSettings.asktosaveonclose = false;
    //InstallConsole(0);
    SIOUXSetTitle("\pTcl Interpreter");

#elif defined(applec)

    /* Init packages used by MPW SIOW package */
    InitGraf((Ptr)&qd.thePort);
    InitFonts();
    InitWindows();
    InitMenus();
    TEInit();
    InitDialogs(nil);
    InitCursor();
       
#endif

    Tcl_MacSetEventProc((Tcl_MacConvertEventPtr) SIOUXHandleOneEvent);

    /* No problems with initialization */
    return TCL_OK;
}
#endif

int appinit(Tcl_Interp *interp) {
/*
    char *buf;
    buf = getenv("Qbic_Home");
    if(buf) printf("buf=%s\n", buf);
*/

    if (Tcl_Init(interp) == TCL_ERROR) return TCL_ERROR;

//  if(buf) Tcl_SetVar2(interp, "tcl_qbic", "Qbic_Home", buf, TCL_GLOBAL_ONLY);

#ifdef _WIN32
    TCLCREATE(SoundPlay);
    TCLCREATE(SoundStop);
#endif
#ifndef __MWERKS__
    TCLCREATE(start_browser);
    TCLCREATE(get_host_ip);
#endif
    TCLCREATE(qbic_clock);
    TCLCREATE(qbic_start); 
    TCLCREATE(qbic_get_param);
    TCLCREATE(qbic_key_fdata);
    TCLCREATE(qbic_image_fdata);
    TCLCREATE(qbic_picker_fdata);
    TCLCREATE(qbic_string_fdata);
    TCLCREATE(qbic_add_feature);
    TCLCREATE(qbic_delete_feature);
    TCLCREATE(qbic_add_image);
    TCLCREATE(qbic_delete_image);
    TCLCREATE(qbic_list_all_features);
    TCLCREATE(qbic_list_cat_features);
    TCLCREATE(qbic_get_cat_records);
    TCLCREATE(qbic_query_image);
    TCLCREATE(qbic_query_key);
    TCLCREATE(qbic_set_keys_return);
    TCLCREATE(qbic_make_connect);
    TCLCREATE(qbic_make_thumb);
    TCLCREATE(qbic_set_keyword);
    TCLCREATE(qbic_set_thumb_24color);
    TCLCREATE(qbic_set_thumb_size);
    TCLCREATE(qbic_set_thumb_pad);
    TCLCREATE(QbMkDbs);
    TCLCREATE(QbQBE);
    TCLCREATE(QbMkThmb);
    TCLCREATE(QbDumpDb);
    TCLCREATE(qbic_end);

    return TCL_OK;
}
