#!/usr/local/qbic/bin/tclsh

### (C) COPYRIGHT International Business Machines Corp. 1998
### Author: Xiaoming Zhu (xzhu@almaden.ibm.com)
### based on versions by Tom Breuel and Jim Hafner
#############################################################################
### This is the basic template for most demos we can set up on the web.
### See template-demo.tcl for the user customizable stuff
### You should be able to override anything here in template-demo.tcl
#############################################################################
# the URL under which images are accessible from the server
set urls(image_prefix)    /images/$params(qbic_catalog)/
# the URL prefix and suffix where thumbnails are accessible from the server
set urls(thumbnail_prefix) /thumb/$params(qbic_catalog)/

set urls(classes) /classes
###########################################################################

# URL prefix for image components (spacers, icons, etc.)
set urls(gifs) /gifs

# URL prefix and suffix for obtaining additional information on an image.
set urls(info_prefix) "$urls(script)/info?"
set urls(info_suffix) ""
set urls(search_prefix) "$urls(script)/search?"

# parameters for the result formatting

set params(qbic_maxresults) 48
set params(images_per_row) 4
set params(qbic_table_images) 12

# QBIC query types available in the HTML-based user interface
## the following initialization is global.  If we add feature classes
## to the demo setup, we can add a stanza here for the new feature class
##    type        maps a QbFeatureClass name to nicer name
##    icon        maps a QbFeatureClass name to an letter icon (image)
##    iconhelp    maps to the text that appears next to the icon in Usage message
##    fname       maps to what is posted in the url query string
##    feature     inverts the fname map
## these next two deal with what's in the "Goto:" section of the page
##    pickerpage  is the 'set page($pickerpage)' resolved in the picker url
##    pickertext  is give text for the picker url 

## Info is just added for uniform treatment of buttons, etc.
set type(QbInfo) "Info"
set icon(QbInfo) "letterI.gif"
set iconhelp(QbInfo) "Get Info"
set fname(QbInfo) "Info"
set feature(Info) "QbInfo"
set pickerpage(QbInfo) ""
set pickertext(QbInfo) ""

set type(QbColorFeatureClass) "Average Color"
set icon(QbColorFeatureClass) "letterA.gif"
set iconhelp(QbColorFeatureClass) "Find Similar Color"
set fname(QbColorFeatureClass) "Average+Color"
set feature(Average+Color) "QbColorFeatureClass"
set pickerpage(QbColorFeatureClass) "colorpicker"
set pickertext(QbColorFeatureClass) "Custom&nbsp;Color&nbsp;Query"

set type(QbColorHistogramFeatureClass) "Color Histogram"
set icon(QbColorHistogramFeatureClass) "letterC.gif"
set iconhelp(QbColorHistogramFeatureClass) "Find Similar Colors"
set fname(QbColorHistogramFeatureClass) "Color+Histogram"
set feature(Color+Histogram) "QbColorHistogramFeatureClass"
set pickerpage(QbColorHistogramFeatureClass) "histpicker"
set pickertext(QbColorHistogramFeatureClass) "Custom&nbsp;Color&nbsp;%&nbsp;Query"

set type(QbDrawFeatureClass) "Color Layout"
set icon(QbDrawFeatureClass) "letterL.gif"
set iconhelp(QbDrawFeatureClass) "Find Similar Layout"
set fname(QbDrawFeatureClass) "Color+Layout"
set feature(Color+Layout) "QbDrawFeatureClass"
set pickerpage(QbDrawFeatureClass) "drawpicker"
set pickertext(QbDrawFeatureClass) "Custom&nbsp;Paint&nbsp;Query"

set type(QbTextureFeatureClass) "Texture"
set icon(QbTextureFeatureClass) "letterT.gif"
set iconhelp(QbTextureFeatureClass) "Find Similar Texture"
set fname(QbTextureFeatureClass) "Texture"
set feature(Texture) "QbTextureFeatureClass"
set pickerpage(QbTextureFeatureClass) ""
#set pickerpage(QbTextureFeatureClass) "texturesampler"
set pickertext(QbTextureFeatureClass) "Texture&nbsp;Samplers"

### END OF PARAMETERS ###
### DON'T REMOVE THE PREVIOUS LINE ###  # why not? #

################################################################
# HTML definitions
################################################################
# complete HTML pages
set page(test) {
    <pre>
    [join [array get env] ""]
    </pre>
}

set page(usage) {
    <html>
    <CENTER><IMG SRC=/gifs/masthead.gif></CENTER>
    <br>

    <table>
    <tr>
    <bf> Goto: </bf> <br>
    <td width=320 valign=top>
    <a href=$urls(script)/start_search> Back to Search Page </a> <br> <p>
    </td>
    <td>
    <H2> Brief Instructions on Using QBIC Demo: </H2>
    In QBIC demo start page, you are presented with a set of randomly
    selected images.  Small Icons such as <img src=/gifs/letterC.gif>
    allow you to choose a different QBIC feature to search for images.
    <ul>
    <li> click <img src=/gifs/letterI.gif> to view the full image and the
         associated keyword list for the image.
    <li> click <img src=/gifs/letterC.gif> to search images with
         similar color percentage.
    <li> click <img src=/gifs/letterL.gif> to search images with
         similar color layout (sensitive to color position).
    <li> click <img src=/gifs/letterT.gif> to search images with
         similar global texture.
    <li> click <img src=/gifs/letterS.gif> to search images with
         special multipass filter (similar to color layout).

    </ul>

    If the images contain keyword information, you can search the
    images with keyword, too.  Enter any keyword in the keyword entry
    box located at the top of the image displaying area.  If you
    simply hit return after entering a keyword, QBIC will present you
    a set of randomly selected images which contain the keyword
    you entered.  If you hit any Icon after entering a keyword, QBIC
    will search the images based on the feature the Icon corresponds
    to and on the keyword you entered.  Keyword search is great to
    constrain your search domain.  But if you want to see all images
    with certain QBIC feature, don't forget to remove any keyword
    in the box.
    </td>
    </tr>
    </table>
    </b>
    </html>
}

set page(drawpicker) {
    [begin_qbic "Draw Query"]

    <center>
    <applet code="DrawQueryApplet.class" codebase="$urls(classes)" width=500 height=360>
    <param name=SUBMIT_URL value=$urls(script)/search>
    <param name=TARGET value=_top>
    [ get_disable_keyword_string ]
    $html(noapplet)
    </applet>
    </center>

    <p> Draw the color layout you are looking for.  Start by clicking
    the mouse in the <em>background</em> of the drawing area then
    dragging out a rectangule of the desired size and shape.  All new
    rectangles are painted white, initially.  Next, select a color
    from the color picker to <em>paint</em> the rectangle.  You can
    either click in the color regions or enter your desired RGB values
    in the text areas.  Repeat this for each rectangle you wish to
    paint.

    <p> Remember, you must start all new rectangles in the background.
    So if you want to add a new rectangle on top of another you have
    already drawn, start the new rectangle in the background and then
    move it or resize it as desired.

    <p> You can <em>repaint</em> any rectangle, <em>move</em> it or
    <em>resize</em> it with the mouse.  With the buttons, you can also
    <em>Delete</em>, <em>Send to Back</em> (put behind all other
    rectangles) or <em>Bring to Front</em> (put in front of all other
    rectangles). Select it with the mouse -- it gets highlighted by
    little corner/side tags and the color picker shows the region's
    color.
    <ul>
    <li>To repaint, pick a new color in the color picker.
    <li>To move it, hold the mouse down and drag the rectangle to the
    desired position.  
    <li>To resize it, drag one of the side/corner tags.
    </ul>
    The <kbd>\[Clear All\]</kbd> button refreshes the drawing area.

    <p> You do not need to paint the entire area. Any unpainted area
    will be viewed as "don't care" by the query.  When you are
    satisfied with your query pattern, click on 
    <kbd>\[Submit Query\]</kbd>.

    <p> Keywords can be added to limit your search to images whose
    Text features match the given words.

    <p> A textual description of the currently selected region appears
    below the drawing.  The format is:
       <blockquote><samp> R(x,y)+(w,h),(r,g,b) </samp></blockquote>
    where <samp>R</samp> means rectangle, the first pair is the
    coordinates of the upper left corner of the rectangle, the second
    pair is the dimensions and the triple is the color in RGB
    coordinates.

    [end_qbic]
}

set page(histpicker) {
    [begin_qbic "Color Histogram Query"]
    
    <center>
    <applet code="HistQueryApplet.class" codebase="$urls(classes)" width=500 height=180>
    <param name=SUBMIT_URL value=$urls(script)/search>
    <param name=TARGET value=_top>
    [ get_disable_keyword_string ]
    $html(noapplet)
    </applet>
    </center>
    
    <p> To specify your color histogram query, "paint" a graphical
    representation of your pattern of colors and approximate
    percentages that you want.

    <p> First select a <em>percentage</em> by clicking in the painting
    area to block off a rectangle of the desired size (percentage of
    the full area).  All new regions are painted white,
    initially. Then select a color from the color picker to
    <em>paint</em> your rectangle. You can either click in the color
    regions or enter your desired RGB values in the text areas. Repeat
    this for each desired color and percentage. You can pick up to 5
    color/percentage pairs.

    <p> You <em>repaint</em> any selection, or <em>resize</em> it
    (adjust the color percentage).  Select a rectangle with the mouse
    -- it gets highlighted with a small black border and the color
    picker shows the region's color.
    <ul>
    <li> To repaint, pick a new color from the color picker.  
    <li> To resize, drag its right edge back and forth.
    </ul>
    <p> Use the <kbd>\[Clear\]</kbd> button to start your query anew.

    <p> You do not need to paint the entire area. Any unpainted area
    will be viewed as "don't care" by the query. When you are
    satisfied with your query pattern, click on 
    <kbd>\[Submit Query\]</kbd> to start the query.

    <p> Keywords can be entered to limit your search to images whose
    Text features match the given words.

    <p> A textual description of your query appears below the drawing.
    It describes, in order, the percentage for each rectangle and the
    color in RGB coordinates.

    [end_qbic]
}

set page(colorpicker) { 
    [begin_qbic "Color Histogram Query"]
    
    <center>
    <applet code="ColorQueryApplet.class" codebase="$urls(classes)" width=300 height=200>
    <param name=SUBMIT_URL value=$urls(script)/search>
    <param name=TARGET value=_top>
    [ get_disable_keyword_string ]
    $html(noapplet)
    </applet>
    </center>

    <p> There are two ways to specify your color query.  You can 
    <ul>
    <li> Click in the "Hue" circle to get the desired hue, then in the
    triangle to get the desired saturation and luminance.  The
    associated RGB values appear in the text areas on the right.
    <li> Type in color values (in range 0...255) for Red, Green and
    Blue in the text fields and hit &LT;enter&GT; (with focus in one
    of the fields). The triangle gets repainted for the associated
    hue.
    </ul>

    <p> The small square at the bottom left shows you a patch of your
    color selection, and a little dot appears in the triangle showing
    you the approximate location of your color.

    <p> When you are satisfied with your color selection, click on 
    <kbd>\[Submit Query\]</kbd> to start the query.

    <p> Keywords can be entered to limit your search to images whose
    Text features match the given words.

    [end_qbic]
}

set page(info_default) {
    <head><title>No Information Available</title></head><body bgcolor=white>
    <h1>No Information Available</h1>
    This QBIC database does not currently store 
    further information on images.
    </body>
}
## this is the default setting, we override it below or in the caller
#set page(info) $page(info_default)

# this is a basic one, others can over ride this 
set page(info) {
    [begin_qbic "Info Page"]
    <head><title>Image Viewer</title></head><body bgcolor=white>
    <br clear=both><img src="$urls(gifs)/sp.gif" height=20><br clear=both>

    <center>
    <h1>Image Name: [get_image_name]</h1>
    Keyword list: [get_keywords]
    <br clear=both><img src="$urls(gifs)/sp.gif" height=20><br clear=both>
    <img src="$urls(image_prefix)[get_image_name]">
    </center>
    </body>
    [end_qbic]
}

set page(info-error) {
    <head><title>No Results for $key</title></head>
    <body bgcolor=white>
    <h1>No Results for $key</h1>
    <p>
    Sorry, but we could not find any additional information on the image.
    <p>
    <hr>
    <p>
    <font size=-2>
    SQL:
    <pre>
    $query(raw)
    </pre>
    </body>
}
    
set page(header) {
    <b>Usage:</b> 
    [ get_header_icon_html ] 
    <p>
}

set page(start_search) {
    [begin_qbic "Search Results"]

    [subst $page(header)]
    
    [process_query -R]
    [make_results_applet 560 470]
    
    <p>[query_info]<p>
    
    [end_qbic]
}

set page(search) {
    [begin_qbic "Search Results"]

    [subst $page(header)]
    
    [process_query]
    [make_results_applet 560 470]
    
    <p>[query_info]<p>
    
    [end_qbic]
}

set page(error) {
    <h1>Error</h1>
    Your request could not be processed due to an error.
    <pre>
    $errorInfo
    </pre>
}

# page parts
set html(top) {
    <html><head><title>$title</title></head>
    <body bgcolor=white>
    <center><img src="$urls(gifs)/masthead.gif"></center>
    <center>
    <table border=0 cellpadding=10>
    <tr>
    <td width=160 Valign="top">
    <img src="$urls(gifs)/sp.gif" width=0 height=70><br clear=both>
    <a href=$urls(script)/usage>
       How to Use QBIC </a> <br> <p>
    <img src="$urls(gifs)/sp.gif" width=0 height=70><br clear=both>
    [subst $params(courtesy) ]
    <br>
    $instructions
    <img src="$urls(gifs)/sp.gif" width=0 height=10><br clear=both>
    <b>Goto:</b><br>
    [subst $html(pickers)]
    <p>
    <a href=http://wwwqbic.almaden.ibm.com/>Official QBIC Page</a>
    </td>
    <td valign=top>
    <img src="$urls(gifs)/sp.gif" width=0 height=100><br clear=both>
    <img src=$urls(gifs)/black.gif width=1 height=300></td>
    <td width=600 valign=center>
}
    
set html(bottom) {    
    </td></tr></table>
    </center>
    </body></html>
}

set html(noapplet) {
    <noapplet>
    <table bgcolor=#ffff88 cellpadding=10><tr><td>
    If your browser supports Java but you have disabled it,
    you can <b>enable it from the menu bar</b>,
    usually under 
    <pre>
        Options &gt; Network Preferences &gt; Languages &gt; Java
    </pre>
    If your browser does not support Java,
    in order to use the Draw Query feature, you need to obtain a copy of
    a browser that does; <a href="http://www.netscape.com/">Netscape 
    Navigator</a> is a good choice.
    </table>
    </noapplet>
}

set html(pickers) { 
    [ get_default_picker_hrefs ]
    <a href=\"$urls(script)/search\">Random&nbsp;Images</a><br> 
} 



################################################################
# scripting
################################################################

set search_results {}

################################################################
#### CGI parameter parsing
################################################################

proc parse_url_query_parameters {array query} {
    upvar $array params
    set fields [split $query {&}]
    foreach field $fields {
        if [regexp {^([^=]*)=(.*)} $field dummy key value] {
            set params($key) $value
        }
    }
}


proc get_feature_table_prefix { feature } {
    global paths 
    regsub {Qb} $feature {} feature
    set result [ string range $feature 0 6 ]
    return ${result}F
}

proc get_image_name {} {
    global env
    catch {set key $env(QUERY_STRING)}
    regsub {\?} $key {} key
    set image [decode_url $key]
    return $image
}

proc get_keywords {} {
    global env paths params
    catch {set key $env(QUERY_STRING)}
    regsub {\?} $key {} key
    set image [decode_url $key]
    set QbQBECMD " -c $params(qbic_catalog) -d $paths(qbic_db) "
    append QbQBECMD " -F QbTextFeatureClass -i $image "
    set klist [eval QbQBE $QbQBECMD]
    regsub {.*QbTextFeatureClass} $klist {} klist
    return $klist
}

proc get_disable_keyword_string {} {
    global params
    if { ![ info exists params(text) ] } {
        return "<param name=disablekeyword value=\"yes\">"
    } else { 
        return ""
    }
}

proc begin_qbic args {
    global qbic_url html title instructions urls Httpd
    set title "Untitled"
    set instructions "<img src=$urls(gifs)/sp.gif width=0 height=100>"
    catch {set title [lindex $args 0]}
#    catch {set instructions [lindex $args 1]}
    return [uplevel {subst $html(top)}]
}

proc end_qbic {} {
    global html
    return [subst $html(bottom)]
}

proc query_info {} {
    global image query cgi env
    set result "<b>Query was:</b><br> "
    append result "<table bgcolor=#ccccff border=0 cellpadding=2>"
    if {$query(raw) == ""} {
        append result [subst {
            <tr><td align=right>Random</td></tr>
        }]
    }
    if {$query(kw) != ""} {
	append result [subst {
            <tr><td align=right>Keywords:</td><td>$query(kw)</td></tr>
        }]
    }
    if {$query(drawquery) != ""} {
	append result [subst {
            <tr><td align=right>Example:</td><td>(from painting)</td></tr>
        }]
    } elseif {$query(image) != ""} {
	append result [subst {
            <tr><td align=right>Example:</td><td>$query(image)</td></tr>
        }]
    }
    if {$query(featurename) != ""} {
	append result [subst {
            <tr><td align=right>Query Type:</td><td>$query(type)</td></tr>
        }]
    }
    append result "</table>"
    return $result
}

proc get_header_icon_html {} {
    global params urls icon iconhelp
    set result {} 
    foreach feature $params(qbefeatures) {
        append result [subst {
            <img src=\"$urls(gifs)/$icon($feature)\">: $iconhelp($feature)
            <img src=\"$urls(gifs)/sp.gif\" width=5>
        }]
    }
    return $result
}

# we use \$image in this so template gets set with this string -
# the next level of subst will resolve this to the correct value
proc get_query_buttons_for_results_table {} {
    global params urls icon fname
    set result {} 
    foreach feature $params(qbefeatures) {
        if { $feature=="QbInfo" } { 
            append result [subst {
                <a href=\"$urls(info_prefix)\$image$urls(info_suffix)\">
                <img src=\"$urls(gifs)/$icon($feature)\" border=0></a>&nbsp;
            }]
        } else {
            append result [subst {
                <a href=\"$urls(search_prefix)fname=$fname($feature)&\$image.x=0\">
                <img src=\"$urls(gifs)/$icon($feature)\" border=0></a>&nbsp;
            }]
        }
    }
    return $result 
}

proc get_default_picker_hrefs {} {
   global params urls pickerpage pickertext
    set result {}
    foreach feature $params(qbefeatures) { 
        set picker $pickerpage($feature) 
        if { $picker != "" } {
            set string $pickertext($feature) 
            append result [subst {<a href=\"$urls(script)/$picker\">$string</a><br>
    }]
        }
    }
    return $result
}

proc decode_url {url} {
    global tcl_platform
    regsub -all {%2F} $url {/} url
    regsub -all {%2E} $url {.} url
    regsub -all {%5F} $url {_} url
    if {$tcl_platform(platform) == "macintosh"} {
       regsub -all {%3A} $url {:} url
    }
    return $url
}

proc parse_query_parameters {} {
    global image query cgi env type feature params

    set query(raw) ""
    set query(image) ""
    set query(featurename) ""
    set query(kw) ""
    set query(drawquery) ""
    set query(type) ""

    catch {parse_url_query_parameters cgi $env(QUERY_STRING)}
    catch {set query(raw) $env(QUERY_STRING)}

    regexp {[?&]([^=]+)\.[xy]=([0-9]+)} $query(raw) dummy query(image)
    set query(image) [decode_url $query(image)]
    
    catch {set query(featurename) $cgi(fname)}
    if { $query(featurename) != "" } {
        set query(featureclass) $feature($query(featurename))
        set query(type) $type($query(featureclass)) 
    } else {
        set query(featureclass) ""
    }
    
    if { [ info exists params(text) ] } { catch {set query(kw) $cgi(kw)} }
    catch {set query(drawquery) $cgi(drawquery)}
    ## if we didn't get anything in the query (probably because it was
    ## a keyword only query, and params(text) is not set), we reset to trigger
    ## random query
    if { $query(image) == "" && $query(kw) == "" && $query(drawquery) == "" } {
        set query(raw) "" 
    }
}

proc remove_weights_from_results {} {
    global search_results
    set temp {}
    foreach item $search_results {
	lappend temp [lindex $item 0]
    }
    set search_results $temp
}

proc make_results_table {max} {
    global search_results query params urls

    if {[llength $search_results] == 0} {
	return "<CENTER><B>The query returned no matches.</B></CENTER>\n"
    }

    # need this subst for get_query_buttons to resolve $image 
    set template {
        <td>
        <img src="$urls(thumbnail_prefix)$image$urls(thumbnail_suffix)" border=0>
        <br clear=both>
        <img src=$urls(gifs)/sp.gif height=2 width=0 border=0>
        <br clear=both>
        [subst [ get_query_buttons_for_results_table ] ]
        </td>
    }
    set result ""
    set i 0
    set total 0
    append result "<table>\n"
    append result "<tr align=center>\n"
    foreach image $search_results {
	if {$total >= $max} break
	if {$i >= $params(images_per_row)} { 
	    append result "</tr><tr align=center>\n" 
	    set i 0
	}
        append result [subst $template]
	incr i
	incr total
    }
    append result "</tr>\n"
    append result "</table>\n"
    return $result
}
proc make_results_applet {w h} {
    global urls params search_results type icon fname query cgi

    set result {} 
    if {[llength $search_results] == 0} {
        append result "<CENTER><B>The query returned no matches.</B></CENTER>\n"
        if { [ info exists query(kw) ] && $query(kw) != "" } {
            append result "<CENTER><B>Perhaps nothing matched your keyword.</B></CENTER>"
        }
        return $result
    }
    
    if { [info exists cgi(kw)] && ![info exists params(text)] } {
        append result "<B>Catalog contains no text data, your keyword $cgi(kw) was ignored!</B>\n"
    }
    append result [subst {
        <applet code=ResultApplet.class codebase="$urls(classes)" width=$w height=$h>
        <param name=submit_url value="$urls(script)/search">
        <param name=thumb_prefix value="$urls(thumbnail_prefix)">
        <param name=thumb_suffix value="$urls(thumbnail_suffix)">
        <param name=info_prefix value="$urls(info_prefix)">
        <param name=info_suffix value="$urls(info_suffix)">
    }]
    ## add this to the other applet pages?
    if { ![ info exists params(text) ]} {
        append result [ get_disable_keyword_string ]
    } elseif { [ info exists query(kw) ] } {
        append result [ subst {
            <param name=keyword value="$query(kw)">
        }]
    }

    set count 0
    foreach feature $params(qbefeatures) {
        append result [subst {
            <param name=qbetype$count value="$fname($feature)">
            <param name=qbeicon$count value="$urls(gifs)/$icon($feature)">
        }]
	incr count
    }
    append result ""
    set count 1
    foreach image $search_results {
	append result "<param name=key$count value=\"$image\">\n"
	incr count
    }
    global params
    append result [subst {
        <noapplet>
        [make_results_table $params(qbic_table_images)]
        (Use a Java-capable Web browser to enable additional functionality.)
        </noapplet>
        </applet>
    }]
    return $result
}


proc qbic_parse_result_exec {tmpresults} {
  # strip distances from the result list
  set i 0
  set results {}
  foreach item $tmpresults {
    if { $i % 2 == 0 } { lappend results $item ; incr i } { incr i }
  }
  return $results
}

proc qbic_qbe_exec { catalog image feature kw nresults } {
  global paths
  # clean off any leading = sign
  regsub {\=} $image {} image

  # puts stdout "Test it Now"
  # convert image to correct -i syntax (or -r)
  if { $image == "random" } { set query "-r" } {
    if { $image == "" } { set query "" } { set query "-i $image" }
  }
  # convert keyword to correct syntax (if not empty)
  if { $kw != "" } { set keyword "-T $kw" } else { set keyword "" }
  # convert feature to correct syntax (if not empty)

  if { $feature == "QbHybridColorFeatureClass" } {
     set featureclass "-f QbColorHistogramFeatureClass:W=.5:O=0"
     append featureclass " -f QbDrawFeatureClass:W=.5:O=1"
  } elseif { $feature == "QbHybrid1ColorFeatureClass" } {
     set featureclass "-f QbColorHistogramFeatureClass:W=.3:O=0"
     append featureclass " -f QbDrawFeatureClass:W=.7:O=1"
  } elseif { $feature != "" } {
     set featureclass "-f $feature"
  } else { set featureclass "" }

  # build up the command string
  set binPath $paths(qbic_bin)
  set QbQBECMD "exec $binPath/QbicMain -c $catalog -d $paths(qbic_db) "
  append QbQBECMD "$query $featureclass -n $nresults $keyword "
  #set fileID [open "$binPath/cmd.bat" w]
  #puts $fileID $QbQBECMD
  #close $fileID

  set tmp [getTMP]
  set tmpresults [eval $QbQBECMD 2>$tmp]
  file delete $tmp
  set results [qbic_parse_result_exec $tmpresults]
  return $results
}

proc qbic_parse_result {tmpresults} {
  # strip distances from the result list
  set i 0
  set results {}
  regsub {.*keys found} $tmpresults {} tmpresults
  foreach item $tmpresults {
    if { $i % 2 == 0 } { lappend results $item }
    incr i
  }
  return $results
}

proc qbic_qbe { catalog image feature kw nresults } {
  global paths
  # clean off any leading = sign
  regsub {\=} $image {} image

  if { $feature == "QbHybridColorFeatureClass" } {
     set featureclass "-f QbColorHistogramFeatureClass:W=.5:O=0"
     append featureclass " -f QbDrawFeatureClass:W=.5:O=1"
  } elseif { $feature == "QbHybrid1ColorFeatureClass" } {
     set featureclass "-f QbColorHistogramFeatureClass:W=.3:O=0"
     append featureclass " -f QbDrawFeatureClass:W=.7:O=1"
  } elseif { $feature != "" } {
     set featureclass "-f $feature"
  } else { set featureclass "" }

  # puts stdout "Test it Now"
  # convert image to correct -i syntax (or -r)
  if { $image == "random" } { set query "-r" } {
    if { $image == "" } { set query "" } { set query "-i $image" }
  }
  # convert keyword to correct syntax (if not empty)
  if { $kw != "" } { set keyword "-T $kw" } { set keyword "" }
  # convert feature to correct syntax (if not empty)

  # build up the command string
  set QbQBECMD " -c $catalog -d $paths(qbic_db) "
  append QbQBECMD "$query $featureclass -n $nresults $keyword "

  set tmpresults [eval QbQBE $QbQBECMD]
  set results [qbic_parse_result $tmpresults]
  return $results
}

proc getTMP {} {
  global tcl_platform
  set os $tcl_platform(platform)
  set fname ""
  switch -exact -- $os {
    windows {set fname "c:/tcltmp.dat"}
    unix    {set fname "/tmp/tcltmp.dat"}
    macintosh {set fname "$env(SYS_FOLDER)/tcltmp.dat"}
  }
  return $fname
}

proc pick_start_images {n, ran} {
  global params paths
  ## get the first feature from the list and use it for getting random keys
  set QbQBECMD " -c $params(qbic_catalog) -d $paths(qbic_db) $ran "
  append QbQBECMD " -n $params(qbic_maxresults)"
  # puts stderr $QbQBECMD
  set tmpresults [ eval QbQBE $QbQBECMD ]
  set results [qbic_parse_result $tmpresults]
  return $results
}

proc runquery {{Out 0}} {
    global cgi env page errorInfo urls html title params instructions
   
    catch {parse_url_query_parameters cgi $env(QUERY_STRING)}
    
    set path ""
    set target "start_search"
    catch {set path $env(PATH_INFO)}
    regexp {/([^./]+)} $path _ target

    #set fileID [open "d:/tmp/cmd.dat" w]
    #if {[info exist env(PATH_INFO)]} {puts $fileID "env=$env(PATH_INFO)"}
    #if {[info exist env(QUERY_STRING)]} {puts $fileID "env=$env(QUERY_STRING)"}
    #puts $fileID "target=$target"
    
    if {[catch {set contents $page($target)}]} {
        set contents $page(search)
    }

    if {[catch {set contents [subst $contents]}]} {
        set contents [subst $page(error)]
    }
    
    #puts $fileID "here ok 1 out=$contents"
    #close $fileID

    if {$Out == 0} {
       set result "Status: 200\n"
       append result "Content-type: text/html\n"
       append result "\n"
       append result $contents
       puts $result 
       return
    } else {
       return $contents
    }
}

proc process_query { {ran -r} } {
    global image env query paths params search_results QbicMain

    parse_query_parameters

    # if there is no query string, output a random set of images.

    if {$query(raw) == ""} {
        #random_seed
        set search_results [pick_start_images $params(qbic_maxresults) $ran]
        return "<!-- random [llength $search_results] -->"
    }

    # build up the command string
    set binPath $paths(qbic_bin)
    if {$query(drawquery) != ""} {
        set QbQBECMD " -c $params(qbic_catalog)"
        append QbQBECMD " -d $paths(qbic_db) -S \"$query(drawquery)\" -f"
        append QbQBECMD " $query(featureclass) -n $params(qbic_maxresults)"
        if { $query(kw) != "" } { append QbQBECMD " -T $query(kw)" }
        set tmpresults [eval QbQBE $QbQBECMD]
        catch {set search_results [qbic_parse_result $tmpresults]}
    } elseif {$query(image) != ""} {
        set query(image) "=$query(image)"
        set search_results [qbic_qbe $params(qbic_catalog) $query(image) \
               $query(featureclass) $query(kw) $params(qbic_maxresults)]
    } elseif {$query(kw) != ""} {
        set search_results [qbic_qbe $params(qbic_catalog) $query(image) \
               $query(featureclass) $query(kw) $params(qbic_maxresults)]
    }
    list
}

###
# if_cgi_string key body missing err
#
# If the CGI parameter exists, execute "body".
# Otherwise, execute "missing".
# The "err" argument is ignored.
# Both the "missing" and "err" arguments are optional.

proc if_cgi_string {key body args} {
    global cgi
    set missing {}
    catch {set missing [lindex $args 0]}
    set err $missing
    catch {set err [lindex $args 1]}
    if {[catch {set value $cgi($key)}]} {uplevel $missing; return}
    if {$value == ""} {uplevel $missing; return}
    uplevel $body
}

###
# if_cgi_int key body missing err
#
# If the CGI parameter exists, execute "body".
# If it cannot be parsed as a number, execute "err".
# Otherwise, execute "missing".
# Both the "missing" and "err" arguments are optional.

proc if_cgi_int {key body args} {
    global cgi
    set missing {}
    catch {set missing [lindex $args 0]}
    set err $missing
    catch {set err [lindex $args 1]}
    if {[catch {set value $cgi($key)}]} {uplevel $missing; return}
    if {![regexp {^-?[0-9]+$} $value]} {uplevel $err; return}
    uplevel $body
}

proc cgi_string {key} {
    global cgi
    set value $cgi($key)
    if {$value == ""} {error "no cgi parameter for $key"}
    return $value
}

proc cgi_number {key} {
    global cgi
    set value $cgi($key)
    if {$value == ""} {error "no cgi parameter for $key"}
    if {![regexp {^[0-9]+$} $value} {error "cgi parameter $key is not numeric"}
    return $value
}


### Local Variables: ***
### mode: tcl ***
### End: ***
