#!/usr/local/qbic/bin/tclsh

### (C) COPYRIGHT International Business Machines Corp. 1998
### Author: Xiaoming Zhu (xzhu@almaden.ibm.com)
#############################################################################
proc fatal {message} {
    global tcl_platform
    puts "Fatal Error:"
    puts stderr "   $message"
    if {$tcl_platform(platform) != "macintosh" } {
        exit 1
    } else {
        return 1
    }
}

proc vexist args {
    foreach v $args {
        if [catch "uplevel {set $v}"] {
            fatal "variable $v undefined in config file"
        }
    }
}

proc do_or_die {command err} {
    if {[catch {uplevel $command}]} { fatal $err }
}

proc get_first_feature_class {} {
    global params paths
    set QbicMainCMD "QbQBE -c $params(qbic_catalog) -d $paths(qbic_db) -l"
    set allfeatures [eval $QbicMainCMD]
    set featureclass [ lindex $allfeatures 0 ] 
    return $featureclass
}

#
# removes the distance information from qbic returned results
#
proc qbic_parse_result {tmpresults} {
  # strip distances from the result list
  set i 0
  set results {}
  foreach item $tmpresults {
    if { $i % 2 == 0 } { lappend results $item ; incr i } { incr i }
  }
  return $results
}

#
# Give image keys the database for the given feature
#
proc qbic_list_keys { featureclass } {
    global params paths
    set QbicMainCMD "QbQBE -c $params(qbic_catalog)"
    append QbicMainCMD " -d $paths(qbic_db) -f $featureclass -r -n 100000000"
    set tmpresults [eval $QbicMainCMD]
    set images [qbic_parse_result $tmpresults]
    return $images
}

#
# clean all files associated with a given catalog: $cat
#
proc QbClean {cat db} {
   set dir [pwd]
   cd $db
   #set platform $tcl_platform(platform)
   catch [set files [glob *.$cat]]
   if {![info exists files]} return
   foreach file $files {
     puts stdout $file
     file delete $file
   }
   cd $dir
}

proc set_config {inputarg} {
   global env params paths root errorInfo urls
   do_or_die {
      set fd [open $inputarg]
      set config [read $fd]
      close $fd
   } "Can't read $inputarg"

   if ![regsub "### END OF PARAMETERS ###.*" $config {} config] {
      fatal "cannot find string '### END OF PARAMETERS ###' in $inputarg"
   }

   do_or_die {
      eval $config
   } "error evaluating config section of $inputarg\n\n$errorInfo"
}

proc walk_file_tree {root var body} {
    global tcl_platform
    upvar $var file
    if {$tcl_platform(platform) == "macintosh" } {
      set sep :
    } else {
      set sep /
    }
    foreach t [glob -nocomplain "$root$sep*"] {
        if [file isdirectory $t] {
            puts stdout $t
            uplevel [list walk_file_tree $t $var $body]
        } else {
            set file $t
            uplevel $body
        }
    }
}

proc GetImageKeyProc {imgData rootdir} {
   global params
   set fileID [open $imgData w]
   foreach root $rootdir {
      puts stdout $root
      walk_file_tree $root filename {
        if {![file isfile $filename]} continue

        set file_ext [file extension $filename]

        if {[regexp -nocase $params(image_re) $file_ext]} {
           if {![regsub "^$paths(image_root)/*" $filename {} key]} {
              puts stderr "something went wrong with the pathnames"
              exit 1
           }
           puts $fileID "$key"
        }
      }
   }
   close $fileID
}
#
# macintosh needs special attention
#
proc FixMacPathIndex {mixedPath} {
    global tcl_platform paths
    set currentDIR $paths(root)

    if { $tcl_platform(platform) != "macintosh" } {
        return $mixedPath
    }


    if { [string range $currentDIR 0 0] == "/"} {
       set currentDIR [string range $currentDIR 1 end]
    }

    if { [string range $mixedPath 0 0] == "/"} {
       set mixedPath [string range $mixedPath 1 end]
    }
 
    regsub $currentDIR $mixedPath {} relPath
    regsub -all "/" $relPath {:} fixed
    set fixedPath $currentDIR
    append fixedPath $fixed
    return $fixedPath
}

proc QbMkDbsProc {imgName cat db bin features img} {
   global paths
   set QbMkDbsCMD "QbMkDbs -c $cat -d $db"
   foreach feature $features {
        append QbMkDbsCMD " -f $feature"
   }
   append QbMkDbsCMD " -e QbicTextPath=$paths(text)"
   append QbMkDbsCMD " $imgName"

   catch {eval $QbMkDbsCMD}
}

proc RemoveLastPath {absolutePath} {
   set mylist [file split $absolutePath]
   set plen [llength $mylist ]
   if {$plen == 1} {
      return :
   } else {
      set absolutePath [lindex $mylist 0]
      for {set i 1} {$i < [expr [llength $mylist] - 1]} {incr i} {
        append absolutePath [lindex $mylist $i]
        append absolutePath :
      }
      return $absolutePath
   }
}

proc MakeAbsolute {relative} {
   global tcl_platform
   if {[file pathtype $relative] != "relative"} {
      return $relative
   }
   set currentDIR [pwd]
   if { $tcl_platform(platform) != "macintosh" } {
      return "[pwd]/$relative"
   }
   if [regsub "^../" $relative {} relative] {
     set currentDIR [ RemoveLastPath $currentDIR ]
   }
   while {[ regsub "^./" $relative {} relative ]} {}
   regsub -all "/" $relative ":" relative
   append currentDIR $relative
   return $currentDIR
}

proc MakeDataBase {} {
   global env params paths feature_classes tcl_platform errorInfo

   puts stdout {
      ################################################################
      # make the qbic database                                       #
      ################################################################
   }

   set currentDir [pwd]

   # make sure the path is absolute; otherwise, the "cd" below will fail

   if {[file pathtype $paths(qbic_db)] == "relative"} {
      set paths(qbic_db) [MakeAbsolute $paths(qbic_db)]
   }
   set paths(qbic_db) [FixMacPathIndex $paths(qbic_db)]
   set paths(image_root) [FixMacPathIndex $paths(image_root)]
   set paths(text) [FixMacPathIndex $paths(text)]

   # create the database

   if {[file exists $paths(qbic_db)]} {
      puts stderr "The directory $paths(qbic_db) already exists"
   } else {
      do_or_die {
         file mkdir $paths(qbic_db)
      } "could not create directory $paths(qbic_db)"
   }

   set qbh [qbic_start $params(qbic_catalog) $paths(qbic_db) w]

   if {[info exists params(clean)] && $params(clean) == "yes"} {
      QbClean $params(qbic_catalog) $paths(qbic_db)
   }

   if {[file pathtype $paths(image_root)] == "relative"} {
      set paths(image_root) [MakeAbsolute $paths(image_root)]
   }
   do_or_die {
      cd $paths(image_root)
   } "could not change to directory $paths(image_root)"

   # add the feature classes to the database

   set feature_classes {}
   foreach t $params(qbefeatures) {
     if [regexp FeatureClass $t] {
        qbic_add_feature $qbh $t
     }
   }

   set count 0
  
   set env(QbicImagePath) $paths(image_root)
   set env(QbicTextPath) $paths(text)
   #puts "text path=$paths(text)"
   foreach root $paths(image_root) {
      walk_file_tree $root filename {
        if {![file isfile $filename]} continue

        set file_ext [file extension $filename]
 
        if {[regexp -nocase $params(image_re) $file_ext]} {
           if { $tcl_platform(platform) != "macintosh" } {
              if {![regsub "^$paths(image_root)/" $filename {} key]} {
                 puts stderr "something went wrong with the pathnames"
                 exit 1
              }
           } else {
              if {![regsub "^$paths(image_root):" $filename {} key]} {
                 puts stderr "something went wrong with the pathnames"
                 exit 1
              }
           }
           incr count

           if [catch {set result [qbic_add_image $qbh $key] } ] {
               puts "error to index $key due to $errorInfo"
           } else {
               puts stdout "<$count> $result"
           }
        }
      }
   }
   qbic_end $qbh
   cd $currentDir
}

proc MakeThumb {} {
   global params paths urls tcl_platform errorInfo
   puts stderr {
       ################################################################
       # make thumbnails                                              #
       ################################################################
   }

   if {[file pathtype $paths(thumbnails)] != "absolute"} { 
      set paths(thumbnails)  [MakeAbsolute $paths(thumbnails)]
   }
   if {[file pathtype $paths(image_root)] != "absolute"} { 
      set paths(image_root)  [MakeAbsolute $paths(image_root)]
   }
   set paths(image_root) [FixMacPathIndex $paths(image_root)]
   set paths(thumbnails) [FixMacPathIndex $paths(thumbnails)]

   catch { file mkdir $paths(thumbnails) }

   set count 0

   set qbh [qbic_start]
   puts "imgRoot=$paths(image_root)"
   walk_file_tree $paths(image_root) imagename {
     puts $imagename
     if {![regexp -nocase $params(image_re) $imagename]} continue
     if {![file isfile $imagename]} continue

     regsub {^\.//*} $imagename {} imagename
     if {$tcl_platform(platform) != "macintosh"} {
        if {[regsub "^$paths(image_root)/*" $imagename {} key]==0} {
           puts stderr "something went wrong with the pathnames"
	         exit 1
        }
     } else {
           if {[regsub "^$paths(image_root)*" $imagename {} key]==0} {
           puts stderr "something went wrong with the pathnames"
	         exit 1
        }
     }

     if {$tcl_platform(platform) != "macintosh"} {
          set out "$paths(thumbnails)/$key$urls(thumbnail_suffix)" 
     } else {
          set out "$paths(thumbnails)$key$urls(thumbnail_suffix)" 
     }
     file mkdir [file dirname $out]
     #set result [qbic_make_thumb $qbh $imagename $out]
     #return
     if {![catch {set result [qbic_make_thumb $qbh $imagename $out]}]} {
        puts "<$count> Thumbnail $out created successfully"
        incr count
     } else {
        puts "error to create $out due to $errorInfo"
     }
   }
   qbic_end $qbh
}

proc MakeCache {} {
   global env params paths
   puts stderr {
       ################################################################
       # make cache                                                   #
       ################################################################
   }

   if {[catch {set features [QbQBE \
          -c $params(qbic_catalog) -d $paths(qbic_db) -l]}]} {
       puts stderr "$params(qbic_catalog) in $paths(qbic_db) not exist"
       return
   }

   foreach feature $features {
#      if {[catch {[QbMkDist \
#            -c $params(qbic_catalog) -d $paths(qbic_db) -F $feature]}]} {
#          puts stderr "Something Wrong here, we better quit"
#          return
#      }
       if { [ string compare $feature "QbTextFeatureClass"] } {
          puts $feature
          exec $paths(qbic_bin)/QbMkDist \
              -c $params(qbic_catalog) -d $paths(qbic_db) -F $feature
       }
   }
}

proc MakeDataBase_Exec {} {
   global env params paths feature_classes

   puts stdout {
      ################################################################
      # make the qbic database                                       #
      ################################################################
   }

   # make sure the path is absolute; otherwise, the "cd" below will fail

   if {[file pathtype $paths(qbic_db)] == "relative"} {
      set paths(qbic_db) "[pwd]/$paths(qbic_db)"
   }

   # create the database

   if {[file exists $paths(qbic_db)]} {
      puts stderr "The directory $paths(qbic_db) already exists"
   } else {
      do_or_die {
         file mkdir $paths(qbic_db)
      } "could not create directory $paths(qbic_db)"
   }

   if {[info exists params(clean)] && $params(clean) == "yes"} {
      QbClean $params(qbic_catalog) $paths(qbic_db)
   }

   do_or_die {
      cd $paths(image_root)
   } "could not change to directory $paths(image_root)"

   # add the feature classes to the database

   set feature_classes {}
   foreach t $params(qbefeatures) {
     if [regexp FeatureClass $t] {
        lappend feature_classes $t
     }
   }

   set currentDir [pwd]
   cd $paths(image_root)

   set count 0
   set imageList {}
   foreach root $paths(image_root) {
      puts stdout $root
      walk_file_tree $root filename {
        if {![file isfile $filename]} continue

        set file_ext [file extension $filename]

        if {[regexp -nocase $params(image_re) $file_ext]} {
           if {![regsub "^$paths(image_root)/*" $filename {} key]} {
              puts stderr "something went wrong with the pathnames"
              exit 1
           }

           incr count
           lappend imageList $key
           if {[expr $count > 50]} {
              QbMkDbsProc $imageList $params(qbic_catalog) $paths(qbic_db) \
                   $paths(qbic_bin) $feature_classes $paths(image_root)
              set imageList {}
              puts stdout "$count image processed"
              set count 0
           }
        }
      }
   }
   if {[llength $imageList]} {
      QbMkDbsProc $imageList $params(qbic_catalog) $paths(qbic_db) \
           $paths(qbic_bin) $feature_classes $paths(image_root)
      puts stdout "$count image processed"
      set imageList {}
   }
}

proc MakeThumb_exec {} {
   global params paths urls
   puts stderr {
       ################################################################
       # make thumbnails                                              #
       ################################################################
   }

   if {[file pathtype $paths(thumbnails)] != "absolute"} { 
      set paths(thumbnails) "[pwd]/$paths(thumbnails)" 
   }

   catch { exec mkdir $paths(thumbnails) }

   set count 0

   walk_file_tree $paths(image_root) imagename {
     if {![regexp -nocase $params(image_re) $imagename]} continue
     if {![file isfile $imagename]} continue

     regsub {^\.//*} $imagename {} imagename
     if {![regsub "^$paths(image_root)/*" $imagename {} key]} {
        puts stderr "something went wrong with the pathnames"
	exit 1
     }

     set out "$paths(thumbnails)/$key$urls(thumbnail_suffix)"
     if {[file exists $out]} { puts stderr "$out: already exists"; continue }

     puts stderr "$key -> $out"

     file mkdir [file dirname $out]
     if {![catch {exec $paths(qbic_bin)/QbMkThmb $imagename $out 128 128 -n -t}]         } {
        incr count
     }
   }
   puts stdout "generated $count thumbnails in $paths(thumbnails)"
}

proc MakeCache_exec {} {
   global env params paths
   puts stderr {
       ################################################################
       # make cache                                                   #
       ################################################################
   }

   if {[catch {set features [exec $paths(qbic_bin)/QbicMain  \
          -c $params(qbic_catalog) -d $paths(qbic_db) -l]}]} {
       puts stderr "$params(qbic_catalog) in $paths(qbic_db) not exist"
       return
   }

   foreach feature $features {
#      if {[catch {[exec $paths(qbic_bin)/QbMkDist \
#            -c $params(qbic_catalog) -d $paths(qbic_db) -F $feature]}]} {
#          puts stderr "Something Wrong here, we better quit"
#          return
#      }
       if { [ string compare $feature "QbTextFeatureClass"] } {
          puts $feature
          exec $paths(qbic_bin)/QbMkDist \
              -c $params(qbic_catalog) -d $paths(qbic_db) -F $feature
       }
   }
}

### Local Variables: ***
### mode: tcl ***
### End: ***
