### Qbic mini http server
###
### Author: Xiaoming Zhu (xzhu@almaden.ibm.com)
### Modified from the work by Stephen Uhler
###########################################################################
# Simple Sample httpd/1.[01] server
# Stephen Uhler (c) 1996-1997 Sun Microsystems

# Httpd is a global array containing the global server state
#  root:	the root of the document directory
#  port:	The port this server is serving
#  listen:	the main listening socket id
#  accepts:	a count of accepted connections so far
#  maxtime:     The max time (msec) allowed to complete an http request
#  maxused:     The max # of requests for a socket

# HTTP/1.[01] error codes (the ones we use)

#set fdbug [open "dbg.dat" w]
set currentDIR [pwd]
if {$tcl_platform(platform) == "macintosh" } {
  regsub -nocase ":bin:" $currentDIR {} currentDIR
} else {
  regsub -nocase "/bin" $currentDIR {} currentDIR
}
puts "qbic root=$currentDIR"

set VirDir(cgi-bin) $currentDIR/script
set VirDir(classes) $currentDIR/classes
set VirDir(public_html) $currentDIR/html
set VirDir(gifs) $currentDIR/html/gifs
set VirDir(thumb) $currentDIR/html/thumb
set VirDir(images) $currentDIR/html/images
set VirDir(docs) $currentDIR/docs

array set HttpdErrors {
    200 {Data Follows}
    204 {No Content}
    400 {Bad Request}
    404 {Not Found}
    408 {Request Timeout}
    411 {Length Required}
    419 {Expectation Failed}
    503 {Service Unavailable}
    504 {Service Temporarily Unavailable}
    }

array set Httpd {
    bufsize	32768
    maxtime	60000000
    maxused	25
}

# Start the server by listening for connections on the desired port.

proc Httpd_Server {root {port 80} {default index.html}} {
    global Httpd env

    set env(sound) 0
    catch {close Httpd(listen)}	;# it might already be running
    array set Httpd [list root $root default $default port $port]
    array set Httpd [list accepts 0 requests 0 errors 0]
    set Httpd(listen) [socket -server HttpdAccept $port]
    return $Httpd(listen)
}

# Accept a new connection from the server and set up a handler
# to read the request from the client.

proc HttpdAccept {sock ipaddr {port {}}} {
    global Httpd
    upvar #0 Httpd$sock data

    incr Httpd(accepts)
    HttpdReset $sock $Httpd(maxused)
    Httpd_Log $sock Connect $ipaddr $port
}

# Initialize or reset the socket state

proc HttpdReset {sock left} {
    global Httpd
    upvar #0 Httpd$sock data

    array set data [list state start linemode 1 version 0 left $left close 0]
    set data(cancel) [after $Httpd(maxtime) [list HttpdTimeout $sock]]
    fconfigure $sock -blocking 0 -buffersize $Httpd(bufsize) \
	-translation {auto crlf}
    fileevent $sock readable [list HttpdRead $sock]
}

# Read data from a client request
# 1) read the request line
# 2) read the mime headers
# 3) read the additional data (if post && content-length not satisfied)

proc HttpdRead {sock} {
    global Httpd env
    upvar #0 Httpd$sock data

    # Use line mode to read the request and the mime headers

    if {$data(linemode)} {
	set readCount [gets $sock line]
	set state [string compare $readCount 0],$data(state)
        Httpd_Log $sock "debug info line=$line state=$state"
	switch -glob -- $state {
	    1,start {
		if {[regexp {(HEAD|POST|GET) ([^?]+)\??([^ ]*) HTTP/1.([01])} \
                   $line x data(proto) data(url) data(query) data(version)]} {
		    set data(state) mime
		    incr Httpd(requests)
		    Httpd_Log $sock Request $data(left) $line $data(query)
		} else {
		    HttpdError $sock 400 $line
		}
	    }
	    0,start {
		Httpd_Log $sock Warning "Initial blank line fetching request"
	    }
	    1,mime {
		if {[regexp {([^:]+):[ 	]*(.*)}  $line {} key value]} {
		    Httpd_Log $sock Info "key=$key value=$value"
		    set key [string tolower $key]
		    set data(key) $key
		    if {[info exists data(mime,$key)]} {
			append data(mime,$key) ", $value"
		    } else {
			set data(mime,$key) $value
		    }
		} elseif {[regexp {^[ 	]+(.+)} $line {} value] && \
			[info exists data(key)]} {
		    append data(mime,$data($key)) " " $value
		} else {
		    HttpdError $sock 400 $line
		}
	    }
	    0,mime {
		Httpd_Log $sock DEBUG "proto=$data(proto)"
	        if {$data(proto) == "POST" && \
	        	[info exists data(mime,content-length)]} {
		    set data(linemode) 0
	            set data(count) $data(mime,content-length)
	            if {$data(version) && [info exists data(mime,expect]} {
	                if {$data(mime,expect) == "100-continue"} {
			    puts $sock "100 Continue HTTP/1.1\n"
			    flush $sock
			} else {
			    HttpdError $sock 419 $data(mime,expect)
			}
		    }
		    fconfigure $sock -translation binary
	        } elseif {$data(proto) != "POST"}  {
		    HttpdRespond $sock
	        } else {
		    HttpdError $sock 411 "Confusing mime headers"
	        }
	    }
	    -1,* {
	    	if {[eof $sock]} {
		    Httpd_Log $sock Error "Broken connection fetching request"
		    HttpdSockDone $sock 1
	    	} else {
	    	    puts stderr "Partial read, retrying"
	    	}
	    }
	    default {
		HttpdError $sock 404 "Invalid http state: $state,[eof $sock]"
	    }
	}

    # Use counted mode to get the post data

    } elseif {![eof $sock]} {
        append data(postdata) [read $sock $data(count)]
        set data(count) [expr {$data(mime,content-length) - \
        	[string length $data(postdata)]}]
        if {$data(count) == 0} {
	    HttpdRespond $sock
	}
    } else {
	Httpd_Log $sock Error "Broken connection reading POST data"
	HttpdSockDone $sock 1
    }
}

# Done with the socket, either close it, or set up for next fetch
#  sock:  The socket I'm done with
#  close: If true, close the socket, otherwise set up for reuse

proc HttpdSockDone {sock close} {
    global Httpd
    upvar #0 Httpd$sock data
    after cancel $data(cancel)
    set left [incr data(left) -1]
    unset data
    if {$close} {
        flush $sock
	close $sock
    } else {
	HttpdReset $sock $left
    }
    return ""
}

# A timeout happened

proc HttpdTimeout {sock} {
    global Httpd
    upvar #0 Httpd$sock data
    HttpdError $sock 408
}

proc HttpdShutdown {sock msg} {
    global Httpd
    HttpdMessage $sock $msg 1
    Httpd_Log $sock "++++++the server stops now+++++++ "
    close $Httpd(listen)
    exit
}

# The following are several routines that return replies

# See if we should close the socket
#  sock:  the connection handle

proc HttpdClose {sock} {
    upvar #0 Httpd$sock data

    if {[info exists data(mime,connection)]} {
        if {[string tolower $data(mime,connection)] == "keep-alive"} {
            set close 0
        } else {
            set close 1
        }
    } elseif {$data(version) >= 1.1} {
        set close 0
    } else {
        # HTTP/1.0
        set close 1
    }
    if {[expr {$data(left) == 0}]} {
        # Exceeded transactions per connection
        set close 1
    }
    return $close
}
#
# macintosh needs special attention
#
proc FixMacPath {mixedPath} {
    global currentDIR tcl_platform
    
    if { $tcl_platform(platform) != "macintosh" } { 
        return $mixedPath 
    }
    regsub $currentDIR $mixedPath {} relPath
    regsub -all "/" $relPath {:} fixed
    set fixedPath $currentDIR
    append fixedPath $fixed
    return $fixedPath
}

# Handle file system queries.  This is a place holder for a more
# generic dispatch mechanism.

proc HttpdRespond {sock} {
    global Httpd HttpdUrlCache cgi params paths urls currentDIR tcl_platform
    upvar #0 Httpd$sock data

    catch {unset params}
    catch {unset paths}
    catch {unset urls}
    catch {unset cgi}

    regsub {(^http://[^/]+)?} $data(url) {} url

    if {![string compare [string tolower $url] "/httpstop"]} {
        HttpdShutdown $sock "Server Shutdown by User"
    }
    
    if {[info exists HttpdUrlCache($url)]} {
    	set mypath $HttpdUrlCache($url)
    } else {
        set mypath [HttpdUrlMap $sock $url]
    }

   
    if { $url == "/" } {
      set url "/cgi-bin/qbicindex.tcl/frontPage/$currentDIR/html/images"
    }

    Httpd_Log $sock DEBUG "mypath=$mypath url=$url"

    set allPath [file split $mypath]
    set rpath [lindex $allPath end]
#
# interprete tcl cgi script
#
    if {![string compare [string tolower [file extension $rpath]] ".tcl"]} {
        set Httpd(close) [HttpdClose $sock]
        HttpdCgiOpen $sock $mypath $rpath
        return
    }
#
# interprete qbicindex.tcl with path info
#
    if {[regexp {/([^\.]*)\.tcl} $url rt cg]} {
        set Httpd(close) [HttpdClose $sock]
        HttpdCgiRun $sock $url $rt $cg
        return
    }

    if {[string length $mypath] == 0} {
	HttpdError $sock 400
    } elseif {![file readable $mypath]} {
	HttpdError $sock 404 $mypath
    } else {
        set close [expr {$data(left) == 0}]
        HttpdRespondHeader $sock [HttpdContentType $mypath] \
            $close [file size $mypath] 200
        ## Should also close socket if recvd connection close header

        if {$close} {
            puts $sock "Connection close:"
        } elseif {$data(version) == 0 && [info exists data(mime,connection)]} {
            if {$data(mime,connection) == "Keep-Alive"} {
                set close 0
                puts $sock "Connection: Keep-Alive"
            }
        }
	puts $sock ""

	if {$data(proto) != "HEAD"} {
            set tmpPath [FixMacPath $mypath]
            #puts "copy $tmpPath to client"
	    set in [open $tmpPath r]
	    fconfigure $in -translation binary
            if {$tcl_platform(platform) == "macintosh"} { ;# mac fcopy broken
               set fileall [read $in]
               close $in
	       fconfigure $sock -translation binary -blocking 0
               SendInPieces $sock $fileall 8192 $close
               return
            }
	    fconfigure $sock -translation binary -blocking 0
	    fconfigure $in -translation binary
	    fcopy $in $sock -command [list HttpdCopyDone $in $sock $close]
	} else {
	    HttpdSockDone $sock $close
	}
    }
}

proc SendInPieces {sock message size close} {
    set maxLen $size
    set msgLen [string length $message]
    set delta [expr $msgLen / $maxLen]
    catch {
        if { [string length $message] > $maxLen} {
           set st 0
           while { $st < $msgLen } {
              set en [expr $st + $maxLen - 1]
              if { $en >= $msgLen } { set en [expr $msgLen - 1] }
              set substr [string range $message $st $en]
	      puts -nonewline $sock $substr
              set st [expr $en + 1]
           }
        } else {
	   puts -nonewline $sock $message
        }
    } reason
    flush $sock
    if {$reason != ""} {puts stderr $reason}
    HttpdSockDone $sock $close
}

# remap url page based on virtual directory setting
proc HttpdUrlMap {sock url} {
    global Httpd HttpdUrlCache VirDir cgi params paths urls currentDIR tcl_platform
    upvar #0 Httpd$sock data
    if {![string compare [string tolower $url] "/"]} {
	 append url "cgi-bin/"
	 append url $Httpd(default)
	 append url "/frontPage/$currentDIR/html/images"
    }
    if {$tcl_platform(platform) == "macintosh"} {
        regsub ":" $url "" tmp
        set dpath [lindex [file split $tmp] 0]
        regsub : $dpath {} dpath
    } else {
        set dpath [lindex [file split $url] 1]
    }
    if [info exist VirDir($dpath)] {
       regsub "/$dpath" $url "" url
       set mypath [HttpdUrl2File $VirDir($dpath) $url]
    } else {
       set mypath [HttpdUrl2File $Httpd(root) $url]
    }
    set mypath [FixMacPath $mypath]
    set HttpdUrlCache($url) $mypath
    return $mypath
}

# open tcl cgi script, interprete it and output result
# Httpd(html)    points to the script
# Httpd(cgi-bin) points to the content of the script
#
proc HttpdCgiOpen {sock script rpath} {
    global Httpd HttpdUrlCache VirDir cgi params paths urls currentDIR
    upvar #0 Httpd$sock data
        set tmpPath [FixMacPath $script]
        if {![catch {set fd [open $tmpPath r]} ]} {
           set myscript [read $fd]
           close $fd
        } else {
           set msg "Can't open catalog $tmpPath <P> please 
               <a href=/cgi-bin/qbicindex.tcl/create/$currentDIR/html/images>
               populate </a> the catalog
           "
           HttpdMessage $sock $msg $Httpd(close)
           return
        }
        set num [regsub "runquery" $myscript "runquery 1" myscript]
        set Httpd($rpath) $script
        set Httpd(result/$rpath) $myscript
        set message [uplevel #0 eval $myscript]
        HttpdMessage $sock $message $Httpd(close)
        return
}

proc HttpdCgiRun {sock url rt cg} {
    global Httpd HttpdUrlCache VirDir cgi params paths env
    upvar #0 Httpd$sock data
    regsub -nocase "/cgi-bin/" $rt {} tclscript
    regsub $rt $url {} op
    set env(PATH_INFO) $op
    set env(QUERY_STRING) $data(query)
    if {[info exists Httpd(result/$tclscript)]} {
       set message [uplevel #0 eval $Httpd(result/$tclscript)]
       HttpdMessage $sock $message $Httpd(close)
    } else {
       set mypath [HttpdUrl2File $Httpd(root) "/script"]
       regsub -nocase "/html" $mypath {} mypath
       HttpdCgiOpen $sock $mypath/$tclscript $tclscript 
    }
    return
}

# Callback when file is done being output to client
# in:  The fd for the file being copied
# sock: The client socket
# close: close the socket if true
# bytes: The # of bytes copied
# error:  The error message (if any)

proc HttpdCopyDone {in sock close bytes {error {}}} {
    global Httpd
    upvar #0 Httpd$sock data
    close $in
    Httpd_Log $sock Done $bytes bytes
    HttpdSockDone $sock $close
}

# convert the file suffix into a mime type
# add your own types as needed

array set HttpdMimeType {
    {}		text/plain
    .txt	text/plain
    .html	text/html
    .htm 	text/html
    .gif	image/gif
    .jpg	image/jpeg
}

proc HttpdContentType {path} {
    global HttpdMimeType

    set type text/plain
    catch {set type $HttpdMimeType([file extension $path])}
    return $type
}

set HttpdMessageFormat {
    <title>Message: %1$s</title>
    <b>%2$s</b>
}

# Generic error response.

set HttpdErrorFormat {
    <title>Error: %1$s</title>
    Got the error: <b>%2$s</b><br>
    while trying to obtain <b>%3$s</b>
}

# Respond with an http message
# sock:  The socket handle to the client
# message:  The message to be posted

proc HttpdMessage {sock message close} {
    upvar #0 Httpd$sock data
    global Httpd HttpdErrors HttpdMessageFormat

    set message [format $HttpdMessageFormat 200 $message]
    set size [string length $message]
    HttpdRespondHeader $sock text/html $close [string length $message] 200
    puts $sock ""
    flush $sock

    fconfigure $sock -translation {auto binary}
    SendInPieces $sock $message 8192 $close
}

# Respond with an error reply
# sock:  The socket handle to the client
# code:  The httpd error code
# args:  Additional information for error logging

proc HttpdError {sock code args} {
    upvar #0 Httpd$sock data
    global Httpd HttpdErrors HttpdErrorFormat

    append data(url) ""
    incr Httpd(errors)
    set message [format $HttpdErrorFormat $code $HttpdErrors($code) $data(url)]
    HttpdRespondHeader $sock text/html 1 [string length $message] $code
    puts $sock ""
    flush $sock

    catch {
	fconfigure $sock  -translation {auto crlf}
	puts -nonewline $sock $message
	flush $sock
    } reason
    HttpdSockDone $sock 1
    Httpd_Log $sock Error $code $HttpdErrors($code) $args $reason
}

# Utility routine for outputting response headers for normal data
# Does not output the end of header markers so additional header lines can be
# added
#  sock:  The connection handle
#  type:  The mime type of this response
#  close: If true, signal connection close headers.  See HttpdClose
#  size:  The size "in bytes" of the response
#  code:   The return code - defualts to 200

proc HttpdRespondHeader {sock type close size {code 200}} {
    global Httpd HttpdErrors
    upvar #0 Httpd$sock data

    set data(code) $code
    append reply "HTTP/$data(version) $code $HttpdErrors($code)" \n
    append reply "Date: [HttpdDate [clock seconds]]" \n
    #append reply "Server: $Httpd(server)\n"

    if {$close} {
        append reply "Connection: Close" \n
    } elseif {$data(version) == 1.0 && !$close} {
        append reply "Connection: Keep-Alive" \n
    }
    append reply "Content-Type: $type" \n
    append reply "Content-Length: $size" \n
    fconfigure $sock -blocking 0 -translation {auto crlf}
    puts -nonewline $sock $reply
    #flush $sock
}

# Generate a date string in HTTP format.

proc HttpdDate {seconds} {
    return [clock format $seconds -format {%a, %d %b %Y %T %Z}]
}

# Log an Httpd transaction.
# This should be replaced as needed.

proc Httpd_Log {sock args} {
    #global fdbug
    #puts $fdbug "LOG: $sock $args"
}

# Convert a url into a pathname. (UNIX version only)
# This is probably not right, and belongs somewhere else.
# - Remove leading http://... if any
# - Collapse all /./ and /../ constructs
# - expand %xx sequences -> disallow "/"'s  and "."'s due to expansions

proc HttpdUrl2File {root url} {
    regsub -all {//+} $url / url		;# collapse multiple /'s
    while {[regsub -all {/\./} $url / url]} {}	;# collapse /./
    while {[regsub -all {/\.\.(/|$)} $url /\x81\\1 url]} {} ;# mark /../
    while {[regsub "/\[^/\x81]+/\x81/" $url / url]} {} ;# collapse /../
    if {![regexp "\x81|%2\[eEfF]" $url]} {	;# invalid /../, / or . ?
	return $root[HttpdCgiMap $url]
    } else {
	return ""
    }
}

# Decode url-encoded strings.

proc HttpdCgiMap {data} {
    regsub -all {([][$\\])} $data {\\\1} data
    regsub -all {%([0-9a-fA-F][0-9a-fA-F])} $data  {[format %c 0x\1]} data
    return [subst $data]
}

proc bgerror {msg} {
    global errorInfo
    puts stderr "bgerror: $msg\n$errorInfo"
}

# set up the url where the web browser should point to
set url "http://"
if [catch {append url [get_host_ip]}] {
   append url "127.0.0.1"
}

# start server at port 3456.  If it is used, use the next one
puts "start port finding "
set port 3456
# fine the next available port
set count 0
while { [catch {set test [socket -server {exit} $port]}] } {
   incr port
   incr count
   if {$count > 500} {
      puts stderr "500 port number tried, something wrong"
      exit
   }
}
close $test
append url ":$port"

#start web browser
puts "start browser at $url"
#if [catch {start_browser $url/cgi-bin/ibm.tcl}] {
if [catch {start_browser $url}] {
   if { $tcl_platform(platform) == "windows" } {
      catch {SoundPlay ../../sound/qerror.wav}
   }
   puts "Can't start netscape, please point your url to the following url"
   puts ""
   puts $url
   puts ""
}

if { $tcl_platform(platform) == "windows" } {
   catch {SoundPlay ../../sound/qbicdemo.wav}
}

set qbicimage ""
Httpd_Server $currentDIR $port qbicindex.tcl/frontPage/$currentDIR/html/images
puts stderr "Starting QBIC httpd server on [info hostname] port $port"
vwait forever		;# start the Tcl event loop
